# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Iterable

import httpx

from ..types import batch_create_params, batch_retrieve_params
from .._types import NOT_GIVEN, Body, Query, Headers, NotGiven
from .._utils import maybe_transform, async_maybe_transform
from .._compat import cached_property
from .._resource import SyncAPIResource, AsyncAPIResource
from .._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from .._base_client import make_request_options
from ..types.batch_create_response import BatchCreateResponse
from ..types.batch_retrieve_response import BatchRetrieveResponse

__all__ = ["BatchResource", "AsyncBatchResource"]


class BatchResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> BatchResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/nextbillion-ai/nextbillion-sdk-python#accessing-raw-response-data-eg-headers
        """
        return BatchResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> BatchResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/nextbillion-ai/nextbillion-sdk-python#with_streaming_response
        """
        return BatchResourceWithStreamingResponse(self)

    def create(
        self,
        *,
        key: str,
        requests: Iterable[batch_create_params.Request] | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> BatchCreateResponse:
        """
        Create Batch Routing

        Args:
          key: A key is a unique identifier that is required to authenticate a request to the
              API.

          requests: An array of objects to collect the details of individual routing queries that
              will form a batch.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._post(
            "/batch",
            body=maybe_transform({"requests": requests}, batch_create_params.BatchCreateParams),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform({"key": key}, batch_create_params.BatchCreateParams),
            ),
            cast_to=BatchCreateResponse,
        )

    def retrieve(
        self,
        *,
        key: str,
        track_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> BatchRetrieveResponse:
        """
        Get Batch Result

        Args:
          key: A key is a unique identifier that is required to authenticate a request to the
              API.

          track_id: Specify the track ID of the batch that was returned in the response after
              submitting a successful batch request.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get(
            "/batch",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "key": key,
                        "track_id": track_id,
                    },
                    batch_retrieve_params.BatchRetrieveParams,
                ),
            ),
            cast_to=BatchRetrieveResponse,
        )


class AsyncBatchResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncBatchResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/nextbillion-ai/nextbillion-sdk-python#accessing-raw-response-data-eg-headers
        """
        return AsyncBatchResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncBatchResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/nextbillion-ai/nextbillion-sdk-python#with_streaming_response
        """
        return AsyncBatchResourceWithStreamingResponse(self)

    async def create(
        self,
        *,
        key: str,
        requests: Iterable[batch_create_params.Request] | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> BatchCreateResponse:
        """
        Create Batch Routing

        Args:
          key: A key is a unique identifier that is required to authenticate a request to the
              API.

          requests: An array of objects to collect the details of individual routing queries that
              will form a batch.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._post(
            "/batch",
            body=await async_maybe_transform({"requests": requests}, batch_create_params.BatchCreateParams),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform({"key": key}, batch_create_params.BatchCreateParams),
            ),
            cast_to=BatchCreateResponse,
        )

    async def retrieve(
        self,
        *,
        key: str,
        track_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> BatchRetrieveResponse:
        """
        Get Batch Result

        Args:
          key: A key is a unique identifier that is required to authenticate a request to the
              API.

          track_id: Specify the track ID of the batch that was returned in the response after
              submitting a successful batch request.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._get(
            "/batch",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform(
                    {
                        "key": key,
                        "track_id": track_id,
                    },
                    batch_retrieve_params.BatchRetrieveParams,
                ),
            ),
            cast_to=BatchRetrieveResponse,
        )


class BatchResourceWithRawResponse:
    def __init__(self, batch: BatchResource) -> None:
        self._batch = batch

        self.create = to_raw_response_wrapper(
            batch.create,
        )
        self.retrieve = to_raw_response_wrapper(
            batch.retrieve,
        )


class AsyncBatchResourceWithRawResponse:
    def __init__(self, batch: AsyncBatchResource) -> None:
        self._batch = batch

        self.create = async_to_raw_response_wrapper(
            batch.create,
        )
        self.retrieve = async_to_raw_response_wrapper(
            batch.retrieve,
        )


class BatchResourceWithStreamingResponse:
    def __init__(self, batch: BatchResource) -> None:
        self._batch = batch

        self.create = to_streamed_response_wrapper(
            batch.create,
        )
        self.retrieve = to_streamed_response_wrapper(
            batch.retrieve,
        )


class AsyncBatchResourceWithStreamingResponse:
    def __init__(self, batch: AsyncBatchResource) -> None:
        self._batch = batch

        self.create = async_to_streamed_response_wrapper(
            batch.create,
        )
        self.retrieve = async_to_streamed_response_wrapper(
            batch.retrieve,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Required, TypedDict

__all__ = ["GeofenceListParams"]


class GeofenceListParams(TypedDict, total=False):
    key: Required[str]
    """
    A key is a unique identifier that is required to authenticate a request to the
    API.
    """

    pn: int
    """Denotes page number.

    Use this along with the `ps` parameter to implement pagination for your searched
    results. This parameter does not have a maximum limit but would return an empty
    response in case a higher value is provided when the result-set itself is
    smaller.
    """

    ps: int
    """Denotes number of search results per page.

    Use this along with the `pn` parameter to implement pagination for your searched
    results.
    """

    tags: str
    """Comma (`,`) separated list of `tags` which will be used to filter the geofences.

    Please note only the geofences which have all the `tags` added to this parameter
    will be included in the result. This parameter can accept a string with a
    maximum length of 256 characters.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional

from .._models import BaseModel

__all__ = ["MapView"]


class MapView(BaseModel):
    east: Optional[str] = None
    """Longitude of the eastern-side of the box."""

    north: Optional[str] = None
    """Longitude of the northern-side of the box."""

    south: Optional[str] = None
    """Longitude of the southern-side of the box."""

    west: Optional[str] = None
    """Longitude of the western-side of the box."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import List
from typing_extensions import Literal

import httpx

from ..._types import NOT_GIVEN, Body, Query, Headers, NotGiven
from ..._utils import maybe_transform, async_maybe_transform
from ..._compat import cached_property
from ..._resource import SyncAPIResource, AsyncAPIResource
from ..._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ..._base_client import make_request_options
from ...types.geofence import console_search_params, console_preview_params
from ...types.geofence.console_search_response import ConsoleSearchResponse
from ...types.geofence.console_preview_response import ConsolePreviewResponse

__all__ = ["ConsoleResource", "AsyncConsoleResource"]


class ConsoleResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> ConsoleResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/nextbillion-ai/nextbillion-sdk-python#accessing-raw-response-data-eg-headers
        """
        return ConsoleResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> ConsoleResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/nextbillion-ai/nextbillion-sdk-python#with_streaming_response
        """
        return ConsoleResourceWithStreamingResponse(self)

    def preview(
        self,
        *,
        type: Literal["`circle`", "`polygon`", "`isochrone`"],
        circle: console_preview_params.Circle | NotGiven = NOT_GIVEN,
        custom_id: str | NotGiven = NOT_GIVEN,
        isochrone: console_preview_params.Isochrone | NotGiven = NOT_GIVEN,
        meta_data: object | NotGiven = NOT_GIVEN,
        name: str | NotGiven = NOT_GIVEN,
        polygon: console_preview_params.Polygon | NotGiven = NOT_GIVEN,
        tags: List[str] | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> ConsolePreviewResponse:
        """
        preview geofence geojson

        Args:
          type: Specify the type of the geofence that is being created.

          circle: Provide the details to create a circular geofence. Please note that this object
              is mandatory when `type` is `circle`. When the `type` is not `circle`, the
              properties of this object will be ignored while creating the geofence.

          custom_id: Set an unique ID for the new geofence. If not provided, an ID will be
              automatically generated in UUID format. A valid `custom_id` can contain letters,
              numbers, "-", & "\\__" only.

              Please note that the ID of a geofence can not be changed once it is created.

          isochrone: Provide the details to create an isochrone based geofence. Use this object when
              `type` is `isochrone`. When the `type` is not `isochrone`, the properties of
              this object will be ignored while creating the geofence.

          meta_data: Metadata of the geofence. Use this field to define custom attributes that
              provide more context and information about the geofence being created like
              country, group ID etc.

              The data being added should be in valid JSON object format (i.e. `key` and
              `value` pairs). Max size allowed for the object is 65kb.

          name: Name of the geofence. Use this field to assign a meaningful, custom name to the
              geofence being created.

          polygon: Provide the details to create a custom polygon type of geofence. Please note
              that this object is mandatory when `type` is `polygon`. When the `type` is not
              `polygon`, the properties of this object will be ignored while creating the
              geofence.

              Self-intersecting polygons or polygons containing other polygons are invalid and
              will be removed while processing the request.

              Area of the polygon should be less than 2000 km<sup>2</sup>.

          tags: An array of strings to associate multiple tags to the geofence. `tags` can be
              used to search or filter geofences (using `Get Geofence List` method).

              Create valid `tags` using a string consisting of alphanumeric characters (A-Z,
              a-z, 0-9) along with the underscore ('\\__') and hyphen ('-') symbols.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._post(
            "/geofence/console/preview",
            body=maybe_transform(
                {
                    "type": type,
                    "circle": circle,
                    "custom_id": custom_id,
                    "isochrone": isochrone,
                    "meta_data": meta_data,
                    "name": name,
                    "polygon": polygon,
                    "tags": tags,
                },
                console_preview_params.ConsolePreviewParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ConsolePreviewResponse,
        )

    def search(
        self,
        *,
        query: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> ConsoleSearchResponse:
        """
        Console Geofence Search API

        Args:
          query: string to be searched, will used to match name or id of geofence.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get(
            "/geofence/console/search",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform({"query": query}, console_search_params.ConsoleSearchParams),
            ),
            cast_to=ConsoleSearchResponse,
        )


class AsyncConsoleResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncConsoleResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/nextbillion-ai/nextbillion-sdk-python#accessing-raw-response-data-eg-headers
        """
        return AsyncConsoleResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncConsoleResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/nextbillion-ai/nextbillion-sdk-python#with_streaming_response
        """
        return AsyncConsoleResourceWithStreamingResponse(self)

    async def preview(
        self,
        *,
        type: Literal["`circle`", "`polygon`", "`isochrone`"],
        circle: console_preview_params.Circle | NotGiven = NOT_GIVEN,
        custom_id: str | NotGiven = NOT_GIVEN,
        isochrone: console_preview_params.Isochrone | NotGiven = NOT_GIVEN,
        meta_data: object | NotGiven = NOT_GIVEN,
        name: str | NotGiven = NOT_GIVEN,
        polygon: console_preview_params.Polygon | NotGiven = NOT_GIVEN,
        tags: List[str] | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> ConsolePreviewResponse:
        """
        preview geofence geojson

        Args:
          type: Specify the type of the geofence that is being created.

          circle: Provide the details to create a circular geofence. Please note that this object
              is mandatory when `type` is `circle`. When the `type` is not `circle`, the
              properties of this object will be ignored while creating the geofence.

          custom_id: Set an unique ID for the new geofence. If not provided, an ID will be
              automatically generated in UUID format. A valid `custom_id` can contain letters,
              numbers, "-", & "\\__" only.

              Please note that the ID of a geofence can not be changed once it is created.

          isochrone: Provide the details to create an isochrone based geofence. Use this object when
              `type` is `isochrone`. When the `type` is not `isochrone`, the properties of
              this object will be ignored while creating the geofence.

          meta_data: Metadata of the geofence. Use this field to define custom attributes that
              provide more context and information about the geofence being created like
              country, group ID etc.

              The data being added should be in valid JSON object format (i.e. `key` and
              `value` pairs). Max size allowed for the object is 65kb.

          name: Name of the geofence. Use this field to assign a meaningful, custom name to the
              geofence being created.

          polygon: Provide the details to create a custom polygon type of geofence. Please note
              that this object is mandatory when `type` is `polygon`. When the `type` is not
              `polygon`, the properties of this object will be ignored while creating the
              geofence.

              Self-intersecting polygons or polygons containing other polygons are invalid and
              will be removed while processing the request.

              Area of the polygon should be less than 2000 km<sup>2</sup>.

          tags: An array of strings to associate multiple tags to the geofence. `tags` can be
              used to search or filter geofences (using `Get Geofence List` method).

              Create valid `tags` using a string consisting of alphanumeric characters (A-Z,
              a-z, 0-9) along with the underscore ('\\__') and hyphen ('-') symbols.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._post(
            "/geofence/console/preview",
            body=await async_maybe_transform(
                {
                    "type": type,
                    "circle": circle,
                    "custom_id": custom_id,
                    "isochrone": isochrone,
                    "meta_data": meta_data,
                    "name": name,
                    "polygon": polygon,
                    "tags": tags,
                },
                console_preview_params.ConsolePreviewParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ConsolePreviewResponse,
        )

    async def search(
        self,
        *,
        query: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> ConsoleSearchResponse:
        """
        Console Geofence Search API

        Args:
          query: string to be searched, will used to match name or id of geofence.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._get(
            "/geofence/console/search",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform({"query": query}, console_search_params.ConsoleSearchParams),
            ),
            cast_to=ConsoleSearchResponse,
        )


class ConsoleResourceWithRawResponse:
    def __init__(self, console: ConsoleResource) -> None:
        self._console = console

        self.preview = to_raw_response_wrapper(
            console.preview,
        )
        self.search = to_raw_response_wrapper(
            console.search,
        )


class AsyncConsoleResourceWithRawResponse:
    def __init__(self, console: AsyncConsoleResource) -> None:
        self._console = console

        self.preview = async_to_raw_response_wrapper(
            console.preview,
        )
        self.search = async_to_raw_response_wrapper(
            console.search,
        )


class ConsoleResourceWithStreamingResponse:
    def __init__(self, console: ConsoleResource) -> None:
        self._console = console

        self.preview = to_streamed_response_wrapper(
            console.preview,
        )
        self.search = to_streamed_response_wrapper(
            console.search,
        )


class AsyncConsoleResourceWithStreamingResponse:
    def __init__(self, console: AsyncConsoleResource) -> None:
        self._console = console

        self.preview = async_to_streamed_response_wrapper(
            console.preview,
        )
        self.search = async_to_streamed_response_wrapper(
            console.search,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from ..._compat import cached_property
from ..._resource import SyncAPIResource, AsyncAPIResource
from .routes.routes import (
    RoutesResource,
    AsyncRoutesResource,
    RoutesResourceWithRawResponse,
    AsyncRoutesResourceWithRawResponse,
    RoutesResourceWithStreamingResponse,
    AsyncRoutesResourceWithStreamingResponse,
)
from .document_templates import (
    DocumentTemplatesResource,
    AsyncDocumentTemplatesResource,
    DocumentTemplatesResourceWithRawResponse,
    AsyncDocumentTemplatesResourceWithRawResponse,
    DocumentTemplatesResourceWithStreamingResponse,
    AsyncDocumentTemplatesResourceWithStreamingResponse,
)

__all__ = ["FleetifyResource", "AsyncFleetifyResource"]


class FleetifyResource(SyncAPIResource):
    @cached_property
    def routes(self) -> RoutesResource:
        return RoutesResource(self._client)

    @cached_property
    def document_templates(self) -> DocumentTemplatesResource:
        return DocumentTemplatesResource(self._client)

    @cached_property
    def with_raw_response(self) -> FleetifyResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/nextbillion-ai/nextbillion-sdk-python#accessing-raw-response-data-eg-headers
        """
        return FleetifyResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> FleetifyResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/nextbillion-ai/nextbillion-sdk-python#with_streaming_response
        """
        return FleetifyResourceWithStreamingResponse(self)


class AsyncFleetifyResource(AsyncAPIResource):
    @cached_property
    def routes(self) -> AsyncRoutesResource:
        return AsyncRoutesResource(self._client)

    @cached_property
    def document_templates(self) -> AsyncDocumentTemplatesResource:
        return AsyncDocumentTemplatesResource(self._client)

    @cached_property
    def with_raw_response(self) -> AsyncFleetifyResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/nextbillion-ai/nextbillion-sdk-python#accessing-raw-response-data-eg-headers
        """
        return AsyncFleetifyResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncFleetifyResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/nextbillion-ai/nextbillion-sdk-python#with_streaming_response
        """
        return AsyncFleetifyResourceWithStreamingResponse(self)


class FleetifyResourceWithRawResponse:
    def __init__(self, fleetify: FleetifyResource) -> None:
        self._fleetify = fleetify

    @cached_property
    def routes(self) -> RoutesResourceWithRawResponse:
        return RoutesResourceWithRawResponse(self._fleetify.routes)

    @cached_property
    def document_templates(self) -> DocumentTemplatesResourceWithRawResponse:
        return DocumentTemplatesResourceWithRawResponse(self._fleetify.document_templates)


class AsyncFleetifyResourceWithRawResponse:
    def __init__(self, fleetify: AsyncFleetifyResource) -> None:
        self._fleetify = fleetify

    @cached_property
    def routes(self) -> AsyncRoutesResourceWithRawResponse:
        return AsyncRoutesResourceWithRawResponse(self._fleetify.routes)

    @cached_property
    def document_templates(self) -> AsyncDocumentTemplatesResourceWithRawResponse:
        return AsyncDocumentTemplatesResourceWithRawResponse(self._fleetify.document_templates)


class FleetifyResourceWithStreamingResponse:
    def __init__(self, fleetify: FleetifyResource) -> None:
        self._fleetify = fleetify

    @cached_property
    def routes(self) -> RoutesResourceWithStreamingResponse:
        return RoutesResourceWithStreamingResponse(self._fleetify.routes)

    @cached_property
    def document_templates(self) -> DocumentTemplatesResourceWithStreamingResponse:
        return DocumentTemplatesResourceWithStreamingResponse(self._fleetify.document_templates)


class AsyncFleetifyResourceWithStreamingResponse:
    def __init__(self, fleetify: AsyncFleetifyResource) -> None:
        self._fleetify = fleetify

    @cached_property
    def routes(self) -> AsyncRoutesResourceWithStreamingResponse:
        return AsyncRoutesResourceWithStreamingResponse(self._fleetify.routes)

    @cached_property
    def document_templates(self) -> AsyncDocumentTemplatesResourceWithStreamingResponse:
        return AsyncDocumentTemplatesResourceWithStreamingResponse(self._fleetify.document_templates)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional

from pydantic import Field as FieldInfo

from .access import Access
from .address import Address
from .._models import BaseModel
from .contacts import Contacts
from .map_view import MapView
from .position import Position
from .categories import Categories

__all__ = ["GeocodeBatchCreateResponse", "Item", "ItemScoring"]


class ItemScoring(BaseModel):
    field_score: Optional[object] = FieldInfo(alias="fieldScore", default=None)
    """
    A breakdown of how closely individual field of the result matched with the
    provided query `q`.
    """

    query_score: Optional[float] = FieldInfo(alias="queryScore", default=None)
    """
    A score, out of 1, indicating how closely the result matches with the provided
    query `q` .
    """


class Item(BaseModel):
    id: Optional[str] = None
    """The unique identifier for the result item."""

    access: Optional[Access] = None
    """
    An array returning the location coordinates of all the access points of the
    search result.
    """

    address: Optional[Address] = None
    """Postal address of the result item."""

    categories: Optional[List[Categories]] = None
    """The list of categories assigned to this place."""

    contacts: Optional[List[Contacts]] = None
    """Contact information like phone, email or website."""

    distance: Optional[int] = None
    """
    The distance "as the crow flies" from the search center to this result item in
    meters.
    """

    map_view: Optional[MapView] = FieldInfo(alias="mapView", default=None)
    """
    The bounding box enclosing the geometric shape (area or line) that an individual
    result covers. `place` typed results have no `mapView`.
    """

    position: Optional[Position] = None
    """Returns the location coordinates of the result."""

    scoring: Optional[ItemScoring] = None
    """Score of the result. A higher score indicates a closer match."""

    title: Optional[str] = None
    """The localized display name of this result item."""


class GeocodeBatchCreateResponse(BaseModel):
    items: Optional[List[Item]] = None
    """
    The results are presented as a JSON list of candidates in ranked order
    (most-likely to least-likely) based on the matched location criteria.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import List
from typing_extensions import Literal

import httpx

from ..._types import NOT_GIVEN, Body, Query, Headers, NotGiven
from ..._utils import maybe_transform, async_maybe_transform
from ..._compat import cached_property
from ..._resource import SyncAPIResource, AsyncAPIResource
from ..._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ..._base_client import make_request_options
from ...types.skynet import (
    monitor_list_params,
    monitor_create_params,
    monitor_delete_params,
    monitor_update_params,
    monitor_retrieve_params,
)
from ...types.skynet.simple_resp import SimpleResp
from ...types.skynet.metadata_param import MetadataParam
from ...types.skynet.monitor_list_response import MonitorListResponse
from ...types.skynet.monitor_create_response import MonitorCreateResponse
from ...types.skynet.monitor_retrieve_response import MonitorRetrieveResponse

__all__ = ["MonitorResource", "AsyncMonitorResource"]


class MonitorResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> MonitorResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/nextbillion-ai/nextbillion-sdk-python#accessing-raw-response-data-eg-headers
        """
        return MonitorResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> MonitorResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/nextbillion-ai/nextbillion-sdk-python#with_streaming_response
        """
        return MonitorResourceWithStreamingResponse(self)

    def create(
        self,
        *,
        key: str,
        tags: List[str],
        type: Literal["`enter`", "`exit`", "`enter_and_exit`", "`speeding`", "`idle`"],
        cluster: Literal["america"] | NotGiven = NOT_GIVEN,
        custom_id: str | NotGiven = NOT_GIVEN,
        description: str | NotGiven = NOT_GIVEN,
        geofence_config: monitor_create_params.GeofenceConfig | NotGiven = NOT_GIVEN,
        geofence_ids: List[str] | NotGiven = NOT_GIVEN,
        idle_config: monitor_create_params.IdleConfig | NotGiven = NOT_GIVEN,
        match_filter: monitor_create_params.MatchFilter | NotGiven = NOT_GIVEN,
        meta_data: MetadataParam | NotGiven = NOT_GIVEN,
        name: str | NotGiven = NOT_GIVEN,
        speeding_config: monitor_create_params.SpeedingConfig | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> MonitorCreateResponse:
        """
        Create a Monitor

        Args:
          key: A key is a unique identifier that is required to authenticate a request to the
              API.

          tags: Use this parameter to add `tags` to the `monitor`. `tags` can be used for
              filtering monitors in the _Get Monitor List_ operation. They can also be used
              for easy identification of monitors.

              Please note that valid `tags` are strings, consisting of alphanumeric characters
              (A-Z, a-z, 0-9) along with the underscore ('\\__') and hyphen ('-') symbols.

          type: Specify the type of activity the `monitor` would detect.

              The `monitor` will be able to detect the specified `type` of activity and create
              events for eligible `asset`. A `monitor` can detect following types of asset
              activity:

              - `enter`: The `monitor` will create an event when a linked `asset` enters into
                the specified geofence.

              - `exit`: The `monitor` will create an event when a linked `asset` exits the
                specified geofence.

              - `enter_and_exit`: The `monitor` will create an event when a linked `asset`
                either enters or exits the specified geofence.

              - `speeding`: The `monitor` will create an event when a linked `asset` exceeds a
                given speed limit.

              - `idle`: The `monitor` will create an event when a linked `asset` exhibits idle
                activity.

              Please note that `assets` and geofences can be linked to a `monitor` using the
              `match_filter` and `geofence_config` attributes respectively.

          cluster: the cluster of the region you want to use

          custom_id: Set a unique ID for the new `monitor`. If not provided, an ID will be
              automatically generated in UUID format. A valid `custom_id` can contain letters,
              numbers, "-", & "\\__" only.

              Please note that the ID of an `monitor` can not be changed once it is created.

          description: Add a description for your `monitor` using this parameter.

          geofence_config: Geofences are geographic boundaries surrounding an area of interest.
              `geofence_config` is used to specify the geofences for creating `enter` or
              `exit` type of events based on the asset's location. When an asset associated
              with the `monitor` enters the given geofence, an `enter` type event is created,
              whereas when the asset moves out of the geofence an `exit` type event is
              created.

              Please note that this object is mandatory when the monitor `type` belongs to one
              of `enter`, `exit` or `enter_and_exit`.

          geofence_ids: **Deprecated. Please use the `geofence_config` to specify the geofence_ids for
              this monitor.**

              An array of strings to collect the geofence IDs that should be linked to the
              `monitor`. Geofences are geographic boundaries that can be used to trigger
              events based on an asset's location.

          idle_config: `idle_config` is used to set up constraints for creating idle events. When an
              asset associated with the `monitor` has not moved a given distance within a
              given time, the Live Tracking API can create events to denote such instances.
              Please note that this object is mandatory when the monitor `type` is `idle`.

              Let's look at the properties of this object.

          match_filter: This object is used to identify the asset(s) on which the `monitor` would be
              applied.

          meta_data: Any valid json object data. Can be used to save customized data. Max size is
              65kb.

          name: Name of the `monitor`. Use this field to assign a meaningful, custom name to the
              `monitor` being created.

          speeding_config: `speeding_config` is used to set up constraints for creating over-speed events.
              When an `asset` associated with a `monitor` is traveling at a speed above the
              given limits, the Live Tracking API can create events to denote such instances.
              There is also an option to set up a tolerance before creating an event. Please
              note that this object is mandatory when `type=speeding`.

              Let's look at the properties of this object.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._post(
            "/skynet/monitor",
            body=maybe_transform(
                {
                    "tags": tags,
                    "type": type,
                    "custom_id": custom_id,
                    "description": description,
                    "geofence_config": geofence_config,
                    "geofence_ids": geofence_ids,
                    "idle_config": idle_config,
                    "match_filter": match_filter,
                    "meta_data": meta_data,
                    "name": name,
                    "speeding_config": speeding_config,
                },
                monitor_create_params.MonitorCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "key": key,
                        "cluster": cluster,
                    },
                    monitor_create_params.MonitorCreateParams,
                ),
            ),
            cast_to=MonitorCreateResponse,
        )

    def retrieve(
        self,
        id: str,
        *,
        key: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> MonitorRetrieveResponse:
        """
        Get a Monitor

        Args:
          key: A key is a unique identifier that is required to authenticate a request to the
              API.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._get(
            f"/skynet/monitor/{id}",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform({"key": key}, monitor_retrieve_params.MonitorRetrieveParams),
            ),
            cast_to=MonitorRetrieveResponse,
        )

    def update(
        self,
        id: str,
        *,
        key: str,
        description: str | NotGiven = NOT_GIVEN,
        geofence_config: monitor_update_params.GeofenceConfig | NotGiven = NOT_GIVEN,
        geofence_ids: List[str] | NotGiven = NOT_GIVEN,
        idle_config: monitor_update_params.IdleConfig | NotGiven = NOT_GIVEN,
        match_filter: monitor_update_params.MatchFilter | NotGiven = NOT_GIVEN,
        meta_data: MetadataParam | NotGiven = NOT_GIVEN,
        name: str | NotGiven = NOT_GIVEN,
        speeding_config: monitor_update_params.SpeedingConfig | NotGiven = NOT_GIVEN,
        tags: List[str] | NotGiven = NOT_GIVEN,
        type: Literal["`enter`", "`exit`", "`enter_and_exit`", "`speeding`", "`idle`"] | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> SimpleResp:
        """
        Update a Monitor

        Args:
          key: A key is a unique identifier that is required to authenticate a request to the
              API.

          description: Use this parameter to update the `description` of the `monitor`.

          geofence_config: `geofence_config` is used to update the set of geofences linked to the `monitor`
              for creating `enter` or `exit` type of events based on the asset's location.
              Please note that this object is mandatory when the monitor `type` belongs to one
              of `enter`, `exit` or `enter_and_exit`.

          geofence_ids: Use this parameter to update the geofences linked to the `monitor` by providing
              the geofence `id` as `,` separated strings. Geofences are geographic boundaries
              that can be used to trigger events based on an asset's location.

          idle_config: `idle_config` is used to update the constraints for creating idle events. When
              an asset associated with the `monitor` has not moved a given distance within a
              given time, the Live Tracking API can create events to denote such instances.

              Please note that this object is mandatory when the monitor `type` is `idle`.

          match_filter: Use this object to update the `attributes` of the `monitor`. Please note that
              using this property will overwrite the existing `attributes` that the monitor
              might be using currently to match any asset(s).

          meta_data: Any valid json object data. Can be used to save customized data. Max size is
              65kb.

          name: Use this parameter to update the `name` of the `monitor`. Users can add
              meaningful names to the monitors like "warehouse_exit", "depot_entry" etc.

          speeding_config: `speeding_config` is used to update the tolerance values for creating over-speed
              events. When an asset associated with a `monitor` is traveling at a speed above
              the given limits, Live Tracking API creates events to indicate such instances.

              Please note that this object is mandatory when the monitor `type` is `speeding`.

          tags: Use this parameter to update the `tags` of the `monitor`. `tags` can be used for
              filtering monitors in the _Get Monitor List_ operation. They can also be used
              for easy identification of monitors. Using this parameter overwrites the
              existing `tags` of the monitor.

              Please note that valid `tags` are strings, consisting of alphanumeric characters
              (A-Z, a-z, 0-9) along with the underscore ('\\__') and hyphen ('-') symbols.

          type: Use this parameter to update the `type` of the `monitor`. The `monitor` will be
              able to detect the specified `type` of activity and create events for eligible
              `asset`. A `monitor` can detect following types of asset activity:

              - `enter`: The `monitor` will create an event when a linked `asset` enters into
                the specified geofence.

              - `exit`: The `monitor` will create an event when a linked `asset` exits the
                specified geofence.

              - `enter_and_exit`: The `monitor` will create an event when a linked `asset`
                either enters or exits the specified geofence.

              - `speeding`: The `monitor` will create an event when a linked `asset` exceeds a
                given speed limit.

              - `idle`: The `monitor` will create an event when a linked `asset` exhibits idle
                activity.

              Please note that `assets` and geofences can be linked to a `monitor` using the
              `match_filter` and `geofence_config` attributes respectively.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._put(
            f"/skynet/monitor/{id}",
            body=maybe_transform(
                {
                    "description": description,
                    "geofence_config": geofence_config,
                    "geofence_ids": geofence_ids,
                    "idle_config": idle_config,
                    "match_filter": match_filter,
                    "meta_data": meta_data,
                    "name": name,
                    "speeding_config": speeding_config,
                    "tags": tags,
                    "type": type,
                },
                monitor_update_params.MonitorUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform({"key": key}, monitor_update_params.MonitorUpdateParams),
            ),
            cast_to=SimpleResp,
        )

    def list(
        self,
        *,
        key: str,
        cluster: Literal["america"] | NotGiven = NOT_GIVEN,
        pn: int | NotGiven = NOT_GIVEN,
        ps: int | NotGiven = NOT_GIVEN,
        sort: str | NotGiven = NOT_GIVEN,
        tags: str | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> MonitorListResponse:
        """
        Get Monitor List

        Args:
          key: A key is a unique identifier that is required to authenticate a request to the
              API.

          cluster: the cluster of the region you want to use

          pn: Denotes page number. Use this along with the `ps` parameter to implement
              pagination for your searched results. This parameter does not have a maximum
              limit but would return an empty response in case a higher value is provided when
              the result-set itself is smaller.

          ps: Denotes number of search results per page. Use this along with the `pn`
              parameter to implement pagination for your searched results.

          sort: Provide a single field to sort the results by. Only `updated_at` or `created_at`
              fields can be selected for ordering the results.

              By default, the result is sorted by `created_at` field in the descending order.
              Allowed values for specifying the order are `asc` for ascending order and `desc`
              for descending order.

          tags: `tags` can be used to filter the monitors. Only those monitors which have all
              the `tags` provided here, will be included in the search result. In case
              multiple `tags` need to be specified, use `,` to separate them.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get(
            "/skynet/monitor/list",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "key": key,
                        "cluster": cluster,
                        "pn": pn,
                        "ps": ps,
                        "sort": sort,
                        "tags": tags,
                    },
                    monitor_list_params.MonitorListParams,
                ),
            ),
            cast_to=MonitorListResponse,
        )

    def delete(
        self,
        id: str,
        *,
        key: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> SimpleResp:
        """
        Delete a Monitor

        Args:
          key: A key is a unique identifier that is required to authenticate a request to the
              API.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._delete(
            f"/skynet/monitor/{id}",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform({"key": key}, monitor_delete_params.MonitorDeleteParams),
            ),
            cast_to=SimpleResp,
        )


class AsyncMonitorResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncMonitorResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/nextbillion-ai/nextbillion-sdk-python#accessing-raw-response-data-eg-headers
        """
        return AsyncMonitorResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncMonitorResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/nextbillion-ai/nextbillion-sdk-python#with_streaming_response
        """
        return AsyncMonitorResourceWithStreamingResponse(self)

    async def create(
        self,
        *,
        key: str,
        tags: List[str],
        type: Literal["`enter`", "`exit`", "`enter_and_exit`", "`speeding`", "`idle`"],
        cluster: Literal["america"] | NotGiven = NOT_GIVEN,
        custom_id: str | NotGiven = NOT_GIVEN,
        description: str | NotGiven = NOT_GIVEN,
        geofence_config: monitor_create_params.GeofenceConfig | NotGiven = NOT_GIVEN,
        geofence_ids: List[str] | NotGiven = NOT_GIVEN,
        idle_config: monitor_create_params.IdleConfig | NotGiven = NOT_GIVEN,
        match_filter: monitor_create_params.MatchFilter | NotGiven = NOT_GIVEN,
        meta_data: MetadataParam | NotGiven = NOT_GIVEN,
        name: str | NotGiven = NOT_GIVEN,
        speeding_config: monitor_create_params.SpeedingConfig | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> MonitorCreateResponse:
        """
        Create a Monitor

        Args:
          key: A key is a unique identifier that is required to authenticate a request to the
              API.

          tags: Use this parameter to add `tags` to the `monitor`. `tags` can be used for
              filtering monitors in the _Get Monitor List_ operation. They can also be used
              for easy identification of monitors.

              Please note that valid `tags` are strings, consisting of alphanumeric characters
              (A-Z, a-z, 0-9) along with the underscore ('\\__') and hyphen ('-') symbols.

          type: Specify the type of activity the `monitor` would detect.

              The `monitor` will be able to detect the specified `type` of activity and create
              events for eligible `asset`. A `monitor` can detect following types of asset
              activity:

              - `enter`: The `monitor` will create an event when a linked `asset` enters into
                the specified geofence.

              - `exit`: The `monitor` will create an event when a linked `asset` exits the
                specified geofence.

              - `enter_and_exit`: The `monitor` will create an event when a linked `asset`
                either enters or exits the specified geofence.

              - `speeding`: The `monitor` will create an event when a linked `asset` exceeds a
                given speed limit.

              - `idle`: The `monitor` will create an event when a linked `asset` exhibits idle
                activity.

              Please note that `assets` and geofences can be linked to a `monitor` using the
              `match_filter` and `geofence_config` attributes respectively.

          cluster: the cluster of the region you want to use

          custom_id: Set a unique ID for the new `monitor`. If not provided, an ID will be
              automatically generated in UUID format. A valid `custom_id` can contain letters,
              numbers, "-", & "\\__" only.

              Please note that the ID of an `monitor` can not be changed once it is created.

          description: Add a description for your `monitor` using this parameter.

          geofence_config: Geofences are geographic boundaries surrounding an area of interest.
              `geofence_config` is used to specify the geofences for creating `enter` or
              `exit` type of events based on the asset's location. When an asset associated
              with the `monitor` enters the given geofence, an `enter` type event is created,
              whereas when the asset moves out of the geofence an `exit` type event is
              created.

              Please note that this object is mandatory when the monitor `type` belongs to one
              of `enter`, `exit` or `enter_and_exit`.

          geofence_ids: **Deprecated. Please use the `geofence_config` to specify the geofence_ids for
              this monitor.**

              An array of strings to collect the geofence IDs that should be linked to the
              `monitor`. Geofences are geographic boundaries that can be used to trigger
              events based on an asset's location.

          idle_config: `idle_config` is used to set up constraints for creating idle events. When an
              asset associated with the `monitor` has not moved a given distance within a
              given time, the Live Tracking API can create events to denote such instances.
              Please note that this object is mandatory when the monitor `type` is `idle`.

              Let's look at the properties of this object.

          match_filter: This object is used to identify the asset(s) on which the `monitor` would be
              applied.

          meta_data: Any valid json object data. Can be used to save customized data. Max size is
              65kb.

          name: Name of the `monitor`. Use this field to assign a meaningful, custom name to the
              `monitor` being created.

          speeding_config: `speeding_config` is used to set up constraints for creating over-speed events.
              When an `asset` associated with a `monitor` is traveling at a speed above the
              given limits, the Live Tracking API can create events to denote such instances.
              There is also an option to set up a tolerance before creating an event. Please
              note that this object is mandatory when `type=speeding`.

              Let's look at the properties of this object.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._post(
            "/skynet/monitor",
            body=await async_maybe_transform(
                {
                    "tags": tags,
                    "type": type,
                    "custom_id": custom_id,
                    "description": description,
                    "geofence_config": geofence_config,
                    "geofence_ids": geofence_ids,
                    "idle_config": idle_config,
                    "match_filter": match_filter,
                    "meta_data": meta_data,
                    "name": name,
                    "speeding_config": speeding_config,
                },
                monitor_create_params.MonitorCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform(
                    {
                        "key": key,
                        "cluster": cluster,
                    },
                    monitor_create_params.MonitorCreateParams,
                ),
            ),
            cast_to=MonitorCreateResponse,
        )

    async def retrieve(
        self,
        id: str,
        *,
        key: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> MonitorRetrieveResponse:
        """
        Get a Monitor

        Args:
          key: A key is a unique identifier that is required to authenticate a request to the
              API.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._get(
            f"/skynet/monitor/{id}",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform({"key": key}, monitor_retrieve_params.MonitorRetrieveParams),
            ),
            cast_to=MonitorRetrieveResponse,
        )

    async def update(
        self,
        id: str,
        *,
        key: str,
        description: str | NotGiven = NOT_GIVEN,
        geofence_config: monitor_update_params.GeofenceConfig | NotGiven = NOT_GIVEN,
        geofence_ids: List[str] | NotGiven = NOT_GIVEN,
        idle_config: monitor_update_params.IdleConfig | NotGiven = NOT_GIVEN,
        match_filter: monitor_update_params.MatchFilter | NotGiven = NOT_GIVEN,
        meta_data: MetadataParam | NotGiven = NOT_GIVEN,
        name: str | NotGiven = NOT_GIVEN,
        speeding_config: monitor_update_params.SpeedingConfig | NotGiven = NOT_GIVEN,
        tags: List[str] | NotGiven = NOT_GIVEN,
        type: Literal["`enter`", "`exit`", "`enter_and_exit`", "`speeding`", "`idle`"] | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> SimpleResp:
        """
        Update a Monitor

        Args:
          key: A key is a unique identifier that is required to authenticate a request to the
              API.

          description: Use this parameter to update the `description` of the `monitor`.

          geofence_config: `geofence_config` is used to update the set of geofences linked to the `monitor`
              for creating `enter` or `exit` type of events based on the asset's location.
              Please note that this object is mandatory when the monitor `type` belongs to one
              of `enter`, `exit` or `enter_and_exit`.

          geofence_ids: Use this parameter to update the geofences linked to the `monitor` by providing
              the geofence `id` as `,` separated strings. Geofences are geographic boundaries
              that can be used to trigger events based on an asset's location.

          idle_config: `idle_config` is used to update the constraints for creating idle events. When
              an asset associated with the `monitor` has not moved a given distance within a
              given time, the Live Tracking API can create events to denote such instances.

              Please note that this object is mandatory when the monitor `type` is `idle`.

          match_filter: Use this object to update the `attributes` of the `monitor`. Please note that
              using this property will overwrite the existing `attributes` that the monitor
              might be using currently to match any asset(s).

          meta_data: Any valid json object data. Can be used to save customized data. Max size is
              65kb.

          name: Use this parameter to update the `name` of the `monitor`. Users can add
              meaningful names to the monitors like "warehouse_exit", "depot_entry" etc.

          speeding_config: `speeding_config` is used to update the tolerance values for creating over-speed
              events. When an asset associated with a `monitor` is traveling at a speed above
              the given limits, Live Tracking API creates events to indicate such instances.

              Please note that this object is mandatory when the monitor `type` is `speeding`.

          tags: Use this parameter to update the `tags` of the `monitor`. `tags` can be used for
              filtering monitors in the _Get Monitor List_ operation. They can also be used
              for easy identification of monitors. Using this parameter overwrites the
              existing `tags` of the monitor.

              Please note that valid `tags` are strings, consisting of alphanumeric characters
              (A-Z, a-z, 0-9) along with the underscore ('\\__') and hyphen ('-') symbols.

          type: Use this parameter to update the `type` of the `monitor`. The `monitor` will be
              able to detect the specified `type` of activity and create events for eligible
              `asset`. A `monitor` can detect following types of asset activity:

              - `enter`: The `monitor` will create an event when a linked `asset` enters into
                the specified geofence.

              - `exit`: The `monitor` will create an event when a linked `asset` exits the
                specified geofence.

              - `enter_and_exit`: The `monitor` will create an event when a linked `asset`
                either enters or exits the specified geofence.

              - `speeding`: The `monitor` will create an event when a linked `asset` exceeds a
                given speed limit.

              - `idle`: The `monitor` will create an event when a linked `asset` exhibits idle
                activity.

              Please note that `assets` and geofences can be linked to a `monitor` using the
              `match_filter` and `geofence_config` attributes respectively.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._put(
            f"/skynet/monitor/{id}",
            body=await async_maybe_transform(
                {
                    "description": description,
                    "geofence_config": geofence_config,
                    "geofence_ids": geofence_ids,
                    "idle_config": idle_config,
                    "match_filter": match_filter,
                    "meta_data": meta_data,
                    "name": name,
                    "speeding_config": speeding_config,
                    "tags": tags,
                    "type": type,
                },
                monitor_update_params.MonitorUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform({"key": key}, monitor_update_params.MonitorUpdateParams),
            ),
            cast_to=SimpleResp,
        )

    async def list(
        self,
        *,
        key: str,
        cluster: Literal["america"] | NotGiven = NOT_GIVEN,
        pn: int | NotGiven = NOT_GIVEN,
        ps: int | NotGiven = NOT_GIVEN,
        sort: str | NotGiven = NOT_GIVEN,
        tags: str | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> MonitorListResponse:
        """
        Get Monitor List

        Args:
          key: A key is a unique identifier that is required to authenticate a request to the
              API.

          cluster: the cluster of the region you want to use

          pn: Denotes page number. Use this along with the `ps` parameter to implement
              pagination for your searched results. This parameter does not have a maximum
              limit but would return an empty response in case a higher value is provided when
              the result-set itself is smaller.

          ps: Denotes number of search results per page. Use this along with the `pn`
              parameter to implement pagination for your searched results.

          sort: Provide a single field to sort the results by. Only `updated_at` or `created_at`
              fields can be selected for ordering the results.

              By default, the result is sorted by `created_at` field in the descending order.
              Allowed values for specifying the order are `asc` for ascending order and `desc`
              for descending order.

          tags: `tags` can be used to filter the monitors. Only those monitors which have all
              the `tags` provided here, will be included in the search result. In case
              multiple `tags` need to be specified, use `,` to separate them.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._get(
            "/skynet/monitor/list",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform(
                    {
                        "key": key,
                        "cluster": cluster,
                        "pn": pn,
                        "ps": ps,
                        "sort": sort,
                        "tags": tags,
                    },
                    monitor_list_params.MonitorListParams,
                ),
            ),
            cast_to=MonitorListResponse,
        )

    async def delete(
        self,
        id: str,
        *,
        key: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> SimpleResp:
        """
        Delete a Monitor

        Args:
          key: A key is a unique identifier that is required to authenticate a request to the
              API.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._delete(
            f"/skynet/monitor/{id}",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform({"key": key}, monitor_delete_params.MonitorDeleteParams),
            ),
            cast_to=SimpleResp,
        )


class MonitorResourceWithRawResponse:
    def __init__(self, monitor: MonitorResource) -> None:
        self._monitor = monitor

        self.create = to_raw_response_wrapper(
            monitor.create,
        )
        self.retrieve = to_raw_response_wrapper(
            monitor.retrieve,
        )
        self.update = to_raw_response_wrapper(
            monitor.update,
        )
        self.list = to_raw_response_wrapper(
            monitor.list,
        )
        self.delete = to_raw_response_wrapper(
            monitor.delete,
        )


class AsyncMonitorResourceWithRawResponse:
    def __init__(self, monitor: AsyncMonitorResource) -> None:
        self._monitor = monitor

        self.create = async_to_raw_response_wrapper(
            monitor.create,
        )
        self.retrieve = async_to_raw_response_wrapper(
            monitor.retrieve,
        )
        self.update = async_to_raw_response_wrapper(
            monitor.update,
        )
        self.list = async_to_raw_response_wrapper(
            monitor.list,
        )
        self.delete = async_to_raw_response_wrapper(
            monitor.delete,
        )


class MonitorResourceWithStreamingResponse:
    def __init__(self, monitor: MonitorResource) -> None:
        self._monitor = monitor

        self.create = to_streamed_response_wrapper(
            monitor.create,
        )
        self.retrieve = to_streamed_response_wrapper(
            monitor.retrieve,
        )
        self.update = to_streamed_response_wrapper(
            monitor.update,
        )
        self.list = to_streamed_response_wrapper(
            monitor.list,
        )
        self.delete = to_streamed_response_wrapper(
            monitor.delete,
        )


class AsyncMonitorResourceWithStreamingResponse:
    def __init__(self, monitor: AsyncMonitorResource) -> None:
        self._monitor = monitor

        self.create = async_to_streamed_response_wrapper(
            monitor.create,
        )
        self.retrieve = async_to_streamed_response_wrapper(
            monitor.retrieve,
        )
        self.update = async_to_streamed_response_wrapper(
            monitor.update,
        )
        self.list = async_to_streamed_response_wrapper(
            monitor.list,
        )
        self.delete = async_to_streamed_response_wrapper(
            monitor.delete,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional

from ..._models import BaseModel

__all__ = ["Pagination"]


class Pagination(BaseModel):
    hasmore: Optional[bool] = None
    """
    A boolean value indicating whether there are more items available beyond the
    current page.
    """

    page: Optional[int] = None
    """An integer value indicating the current page number (starting at 0)."""

    size: Optional[int] = None
    """An integer value indicating the maximum number of items retrieved per `page`."""

    total: Optional[int] = None
    """An integer value indicating the total number of items available in the data set.

    This parameter can be used to calculate the total number of pages available.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal

import httpx

from ..types import mdm_create_distance_matrix_params, mdm_get_distance_matrix_status_params
from .._types import NOT_GIVEN, Body, Query, Headers, NotGiven
from .._utils import maybe_transform, async_maybe_transform
from .._compat import cached_property
from .._resource import SyncAPIResource, AsyncAPIResource
from .._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from .._base_client import make_request_options
from ..types.mdm_create_distance_matrix_response import MdmCreateDistanceMatrixResponse
from ..types.mdm_get_distance_matrix_status_response import MdmGetDistanceMatrixStatusResponse

__all__ = ["MdmResource", "AsyncMdmResource"]


class MdmResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> MdmResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/nextbillion-ai/nextbillion-sdk-python#accessing-raw-response-data-eg-headers
        """
        return MdmResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> MdmResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/nextbillion-ai/nextbillion-sdk-python#with_streaming_response
        """
        return MdmResourceWithStreamingResponse(self)

    def create_distance_matrix(
        self,
        *,
        key: str,
        option: Literal["flexible"],
        origins: str,
        spliter: Literal["od_number_spliter", "straight_distance_spliter", "location_spliter"] | NotGiven = NOT_GIVEN,
        area: Literal["singapore", "usa", "india"] | NotGiven = NOT_GIVEN,
        avoid: Literal[
            "`toll`",
            "`ferry`",
            "`highway`",
            "`sharp_turn`",
            "`service_road`",
            "`bbox`",
            "`left_turn`",
            "`right_turn`",
            "`none`",
        ]
        | NotGiven = NOT_GIVEN,
        cross_border: bool | NotGiven = NOT_GIVEN,
        departure_time: int | NotGiven = NOT_GIVEN,
        destinations: str | NotGiven = NOT_GIVEN,
        destinations_approach: Literal["`unrestricted`", "`curb`"] | NotGiven = NOT_GIVEN,
        hazmat_type: Literal["`general`", "`circumstantial`", "`explosive`", "`harmful_to_water`"]
        | NotGiven = NOT_GIVEN,
        mode: Literal["`car`", "`truck`"] | NotGiven = NOT_GIVEN,
        origins_approach: Literal["`unrestricted`", "`curb`"] | NotGiven = NOT_GIVEN,
        route_type: Literal["`fastest`", "`shortest`"] | NotGiven = NOT_GIVEN,
        truck_axle_load: float | NotGiven = NOT_GIVEN,
        truck_size: str | NotGiven = NOT_GIVEN,
        truck_weight: int | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> MdmCreateDistanceMatrixResponse:
        """
        Create a massive distance matrix task

        Args:
          key: A key is a unique identifier that is required to authenticate a request to the
              API.

          option: Use this option to switch to truck-specific routing or time based routing or if
              you want to choose between the fastest and shortest route types.

          origins: `origins` are the starting point of your route. Ensure that origins are routable
              land locations. Multiple origins should be separated by a pipe symbol (|).

              **Format:** latitude_1,longitude_1|latitude_2,longitude_2|…

          spliter:
              Specify a spliter to split the matrix by. It accepts 2 values:

              - `od_number_spliter`:

              - `straight_distance_spliter`:

              Please note it is an internal, debug field only.

              _debug field. choose specific spliter to split matrix._

          area: Provide the country that the coordinates belong to.

              _the input coordinates area._

          avoid: Setting this will ensure the route avoids the object(s) specified as input.
              Multiple values should be separated by a pipe (|). If `none` is provided along
              with other values, an error is returned as a valid route is not feasible.

              - **Note:**

                - This parameter is effective only when `route_type=fastest`.
                - When this parameter is not provided in the input, ferries are set to be
                  avoided by default. When `avoid` input is provided, only the mentioned
                  objects are avoided.
                - When using `avoid=bbox` users also need to specify the boundaries of the
                  bounding box to be avoid. Multiple bounding boxes can be specified
                  simultaneously. Please note that bounding box is a hard filter and if it
                  blocks all possible routes between given locations, a 4xx error is returned.

                  - **Format:** bbox: min_latitude,min_longtitude,max_latitude,max_longitude.
                  - **Example:** avoid=bbox: 34.0635,-118.2547, 34.0679,-118.2478 | bbox:
                    34.0521,-118.2342, 34.0478,-118.2437

                - When using `avoid=sharp_turn`, default range of permissible turn angles is
                  \\[[120,240\\]].

          cross_border: Specify if crossing an international border is expected for operations near
              border areas. When set to false, the API will prohibit routes going back & forth
              between countries. Consequently, routes within the same country will be
              preferred if they are feasible for the given set of `destination` or `waypoints`
              . When set to true, the routes will be allowed to go back & forth between
              countries as needed.

              This feature is available in North America region only. Please get in touch with
              [support@nextbillion.ai](mailto:support@nextbillion.ai) to enquire/enable other
              areas.

          departure_time: This is a number in UNIX epoch timestamp in seconds format that can be used to
              provide the departure time. The response will return the `distance` and
              `duration` of the route based on typical traffic for at the given start time.If
              no input is provided for this parameter then the traffic conditions at the time
              of making the request are considered.

              Please note that when `route_type` is set to `shortest` then the
              `departure_time` will be ineffective as the service will return the result for
              the shortest path possible irrespective of the traffic conditions.

          destinations: `destinations` are the ending coordinates of your route. Ensure that
              destinations are routable land locations. Multiple destinations should be
              separated by a pipe symbol (|).

              In case `destinations` are not provided or if it is left empty, then the input
              value of `origins` will be copied to `destinations` to create the OD matrix
              pairs.

              **Format:** latitude_1,longitude_1|latitude_2,longitude_2|…

          destinations_approach: Specify the side of the road from which to approach `destinations` points.
              Please note that the given approach will be applied to all the `destinations`.

          hazmat_type: Specify the type of hazardous material being carried and the service will avoid
              roads which are not suitable for the type of goods specified. Multiple values
              can be separated using a pipe operator `|` .

              Please note that this parameter is effective only when `mode=truck`.

          mode: Set which driving mode the service should use to determine a route.

              For example, if you use `car`, the API will return a route that a car can take.
              Using `truck` will return a route a truck can use, taking into account
              appropriate truck routing restrictions.

          origins_approach: Specify the side of the road from which to approach `origins` points. Please
              note that the given approach will be applied to all the points provided as
              `origins`.

          route_type: Set the route type that needs to be returned. Please note that `route_type` is
              effective only when `option=flexible`.

          truck_axle_load: Specify the total load per axle (including the weight of trailers and shipped
              goods) of the truck, in tonnes. When used, the service will return routes which
              are legally allowed to carry the load specified per axle.

              Please note this parameter is effective only when `mode=truck`.

          truck_size: This defines the dimensions of a truck in centimeters (cm) in the format of
              "height,width,length". This parameter is effective only when `mode=truck` and
              `option=flexible`. Maximum dimensions are as follows:

              Height = 1000 cm Width = 5000 cm Length = 5000 cm

          truck_weight: This parameter defines the weight of the truck including trailers and shipped
              goods in kilograms (kg). This parameter is effective only when `mode=truck` and
              `option=flexible`.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._post(
            "/mdm/create",
            body=maybe_transform(
                {
                    "origins": origins,
                    "area": area,
                    "avoid": avoid,
                    "cross_border": cross_border,
                    "departure_time": departure_time,
                    "destinations": destinations,
                    "destinations_approach": destinations_approach,
                    "hazmat_type": hazmat_type,
                    "mode": mode,
                    "origins_approach": origins_approach,
                    "route_type": route_type,
                    "truck_axle_load": truck_axle_load,
                    "truck_size": truck_size,
                    "truck_weight": truck_weight,
                },
                mdm_create_distance_matrix_params.MdmCreateDistanceMatrixParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "key": key,
                        "option": option,
                        "spliter": spliter,
                    },
                    mdm_create_distance_matrix_params.MdmCreateDistanceMatrixParams,
                ),
            ),
            cast_to=MdmCreateDistanceMatrixResponse,
        )

    def get_distance_matrix_status(
        self,
        *,
        id: str,
        key: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> MdmGetDistanceMatrixStatusResponse:
        """
        Get massive distance matrix task status

        Args:
          id: Provide the unique ID that was returned on successful submission of the
              Asynchronous Distance Matrix POST request.

          key: A key is a unique identifier that is required to authenticate a request to the
              API.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get(
            "/mdm/status",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "id": id,
                        "key": key,
                    },
                    mdm_get_distance_matrix_status_params.MdmGetDistanceMatrixStatusParams,
                ),
            ),
            cast_to=MdmGetDistanceMatrixStatusResponse,
        )


class AsyncMdmResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncMdmResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/nextbillion-ai/nextbillion-sdk-python#accessing-raw-response-data-eg-headers
        """
        return AsyncMdmResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncMdmResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/nextbillion-ai/nextbillion-sdk-python#with_streaming_response
        """
        return AsyncMdmResourceWithStreamingResponse(self)

    async def create_distance_matrix(
        self,
        *,
        key: str,
        option: Literal["flexible"],
        origins: str,
        spliter: Literal["od_number_spliter", "straight_distance_spliter", "location_spliter"] | NotGiven = NOT_GIVEN,
        area: Literal["singapore", "usa", "india"] | NotGiven = NOT_GIVEN,
        avoid: Literal[
            "`toll`",
            "`ferry`",
            "`highway`",
            "`sharp_turn`",
            "`service_road`",
            "`bbox`",
            "`left_turn`",
            "`right_turn`",
            "`none`",
        ]
        | NotGiven = NOT_GIVEN,
        cross_border: bool | NotGiven = NOT_GIVEN,
        departure_time: int | NotGiven = NOT_GIVEN,
        destinations: str | NotGiven = NOT_GIVEN,
        destinations_approach: Literal["`unrestricted`", "`curb`"] | NotGiven = NOT_GIVEN,
        hazmat_type: Literal["`general`", "`circumstantial`", "`explosive`", "`harmful_to_water`"]
        | NotGiven = NOT_GIVEN,
        mode: Literal["`car`", "`truck`"] | NotGiven = NOT_GIVEN,
        origins_approach: Literal["`unrestricted`", "`curb`"] | NotGiven = NOT_GIVEN,
        route_type: Literal["`fastest`", "`shortest`"] | NotGiven = NOT_GIVEN,
        truck_axle_load: float | NotGiven = NOT_GIVEN,
        truck_size: str | NotGiven = NOT_GIVEN,
        truck_weight: int | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> MdmCreateDistanceMatrixResponse:
        """
        Create a massive distance matrix task

        Args:
          key: A key is a unique identifier that is required to authenticate a request to the
              API.

          option: Use this option to switch to truck-specific routing or time based routing or if
              you want to choose between the fastest and shortest route types.

          origins: `origins` are the starting point of your route. Ensure that origins are routable
              land locations. Multiple origins should be separated by a pipe symbol (|).

              **Format:** latitude_1,longitude_1|latitude_2,longitude_2|…

          spliter:
              Specify a spliter to split the matrix by. It accepts 2 values:

              - `od_number_spliter`:

              - `straight_distance_spliter`:

              Please note it is an internal, debug field only.

              _debug field. choose specific spliter to split matrix._

          area: Provide the country that the coordinates belong to.

              _the input coordinates area._

          avoid: Setting this will ensure the route avoids the object(s) specified as input.
              Multiple values should be separated by a pipe (|). If `none` is provided along
              with other values, an error is returned as a valid route is not feasible.

              - **Note:**

                - This parameter is effective only when `route_type=fastest`.
                - When this parameter is not provided in the input, ferries are set to be
                  avoided by default. When `avoid` input is provided, only the mentioned
                  objects are avoided.
                - When using `avoid=bbox` users also need to specify the boundaries of the
                  bounding box to be avoid. Multiple bounding boxes can be specified
                  simultaneously. Please note that bounding box is a hard filter and if it
                  blocks all possible routes between given locations, a 4xx error is returned.

                  - **Format:** bbox: min_latitude,min_longtitude,max_latitude,max_longitude.
                  - **Example:** avoid=bbox: 34.0635,-118.2547, 34.0679,-118.2478 | bbox:
                    34.0521,-118.2342, 34.0478,-118.2437

                - When using `avoid=sharp_turn`, default range of permissible turn angles is
                  \\[[120,240\\]].

          cross_border: Specify if crossing an international border is expected for operations near
              border areas. When set to false, the API will prohibit routes going back & forth
              between countries. Consequently, routes within the same country will be
              preferred if they are feasible for the given set of `destination` or `waypoints`
              . When set to true, the routes will be allowed to go back & forth between
              countries as needed.

              This feature is available in North America region only. Please get in touch with
              [support@nextbillion.ai](mailto:support@nextbillion.ai) to enquire/enable other
              areas.

          departure_time: This is a number in UNIX epoch timestamp in seconds format that can be used to
              provide the departure time. The response will return the `distance` and
              `duration` of the route based on typical traffic for at the given start time.If
              no input is provided for this parameter then the traffic conditions at the time
              of making the request are considered.

              Please note that when `route_type` is set to `shortest` then the
              `departure_time` will be ineffective as the service will return the result for
              the shortest path possible irrespective of the traffic conditions.

          destinations: `destinations` are the ending coordinates of your route. Ensure that
              destinations are routable land locations. Multiple destinations should be
              separated by a pipe symbol (|).

              In case `destinations` are not provided or if it is left empty, then the input
              value of `origins` will be copied to `destinations` to create the OD matrix
              pairs.

              **Format:** latitude_1,longitude_1|latitude_2,longitude_2|…

          destinations_approach: Specify the side of the road from which to approach `destinations` points.
              Please note that the given approach will be applied to all the `destinations`.

          hazmat_type: Specify the type of hazardous material being carried and the service will avoid
              roads which are not suitable for the type of goods specified. Multiple values
              can be separated using a pipe operator `|` .

              Please note that this parameter is effective only when `mode=truck`.

          mode: Set which driving mode the service should use to determine a route.

              For example, if you use `car`, the API will return a route that a car can take.
              Using `truck` will return a route a truck can use, taking into account
              appropriate truck routing restrictions.

          origins_approach: Specify the side of the road from which to approach `origins` points. Please
              note that the given approach will be applied to all the points provided as
              `origins`.

          route_type: Set the route type that needs to be returned. Please note that `route_type` is
              effective only when `option=flexible`.

          truck_axle_load: Specify the total load per axle (including the weight of trailers and shipped
              goods) of the truck, in tonnes. When used, the service will return routes which
              are legally allowed to carry the load specified per axle.

              Please note this parameter is effective only when `mode=truck`.

          truck_size: This defines the dimensions of a truck in centimeters (cm) in the format of
              "height,width,length". This parameter is effective only when `mode=truck` and
              `option=flexible`. Maximum dimensions are as follows:

              Height = 1000 cm Width = 5000 cm Length = 5000 cm

          truck_weight: This parameter defines the weight of the truck including trailers and shipped
              goods in kilograms (kg). This parameter is effective only when `mode=truck` and
              `option=flexible`.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._post(
            "/mdm/create",
            body=await async_maybe_transform(
                {
                    "origins": origins,
                    "area": area,
                    "avoid": avoid,
                    "cross_border": cross_border,
                    "departure_time": departure_time,
                    "destinations": destinations,
                    "destinations_approach": destinations_approach,
                    "hazmat_type": hazmat_type,
                    "mode": mode,
                    "origins_approach": origins_approach,
                    "route_type": route_type,
                    "truck_axle_load": truck_axle_load,
                    "truck_size": truck_size,
                    "truck_weight": truck_weight,
                },
                mdm_create_distance_matrix_params.MdmCreateDistanceMatrixParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform(
                    {
                        "key": key,
                        "option": option,
                        "spliter": spliter,
                    },
                    mdm_create_distance_matrix_params.MdmCreateDistanceMatrixParams,
                ),
            ),
            cast_to=MdmCreateDistanceMatrixResponse,
        )

    async def get_distance_matrix_status(
        self,
        *,
        id: str,
        key: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> MdmGetDistanceMatrixStatusResponse:
        """
        Get massive distance matrix task status

        Args:
          id: Provide the unique ID that was returned on successful submission of the
              Asynchronous Distance Matrix POST request.

          key: A key is a unique identifier that is required to authenticate a request to the
              API.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._get(
            "/mdm/status",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform(
                    {
                        "id": id,
                        "key": key,
                    },
                    mdm_get_distance_matrix_status_params.MdmGetDistanceMatrixStatusParams,
                ),
            ),
            cast_to=MdmGetDistanceMatrixStatusResponse,
        )


class MdmResourceWithRawResponse:
    def __init__(self, mdm: MdmResource) -> None:
        self._mdm = mdm

        self.create_distance_matrix = to_raw_response_wrapper(
            mdm.create_distance_matrix,
        )
        self.get_distance_matrix_status = to_raw_response_wrapper(
            mdm.get_distance_matrix_status,
        )


class AsyncMdmResourceWithRawResponse:
    def __init__(self, mdm: AsyncMdmResource) -> None:
        self._mdm = mdm

        self.create_distance_matrix = async_to_raw_response_wrapper(
            mdm.create_distance_matrix,
        )
        self.get_distance_matrix_status = async_to_raw_response_wrapper(
            mdm.get_distance_matrix_status,
        )


class MdmResourceWithStreamingResponse:
    def __init__(self, mdm: MdmResource) -> None:
        self._mdm = mdm

        self.create_distance_matrix = to_streamed_response_wrapper(
            mdm.create_distance_matrix,
        )
        self.get_distance_matrix_status = to_streamed_response_wrapper(
            mdm.get_distance_matrix_status,
        )


class AsyncMdmResourceWithStreamingResponse:
    def __init__(self, mdm: AsyncMdmResource) -> None:
        self._mdm = mdm

        self.create_distance_matrix = async_to_streamed_response_wrapper(
            mdm.create_distance_matrix,
        )
        self.get_distance_matrix_status = async_to_streamed_response_wrapper(
            mdm.get_distance_matrix_status,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional

from pydantic import Field as FieldInfo

from .access import Access
from .address import Address
from .._models import BaseModel
from .contacts import Contacts
from .map_view import MapView
from .position import Position
from .categories import Categories

__all__ = [
    "RevgeocodeRetrieveResponse",
    "Item",
    "ItemOpeningHours",
    "ItemOpeningHoursTimeRange",
    "ItemOpeningHoursTimeRangeEndTime",
    "ItemOpeningHoursTimeRangeStartTime",
    "ItemScoring",
]


class ItemOpeningHoursTimeRangeEndTime(BaseModel):
    date: Optional[str] = None
    """The date to which the subsequent closing time details belong to."""

    hour: Optional[int] = None
    """The hour of the day when the place closes."""

    minute: Optional[int] = None
    """The minute of the hour when the place closes."""


class ItemOpeningHoursTimeRangeStartTime(BaseModel):
    date: Optional[str] = None
    """The date to which the subsequent open time details belong to."""

    hour: Optional[int] = None
    """The hour of the day when the place opens."""

    minute: Optional[int] = None
    """The minute of the hour when the place opens."""


class ItemOpeningHoursTimeRange(BaseModel):
    end_time: Optional[ItemOpeningHoursTimeRangeEndTime] = FieldInfo(alias="endTime", default=None)
    """Returns the closing time details."""

    start_time: Optional[ItemOpeningHoursTimeRangeStartTime] = FieldInfo(alias="startTime", default=None)
    """Returns the open time details."""


class ItemOpeningHours(BaseModel):
    time_ranges: Optional[List[ItemOpeningHoursTimeRange]] = FieldInfo(alias="timeRanges", default=None)
    """
    A collection of attributes with details about the opening and closing hours for
    each day of the week.
    """


class ItemScoring(BaseModel):
    field_score: Optional[object] = FieldInfo(alias="fieldScore", default=None)
    """
    A breakdown of how closely individual field of the result matched with the
    provided query `q`.
    """

    query_score: Optional[float] = FieldInfo(alias="queryScore", default=None)
    """
    A score, out of 1, indicating how closely the result matches with the provided
    query `q` .
    """


class Item(BaseModel):
    id: Optional[str] = None
    """The unique identifier for the result item."""

    access: Optional[Access] = None
    """
    An array returning the location coordinates of all the access points of the
    search result.
    """

    address: Optional[Address] = None
    """Postal address of the result item."""

    categories: Optional[List[Categories]] = None
    """The list of categories assigned to this place."""

    contacts: Optional[List[Contacts]] = None
    """Contact information like phone, email or website."""

    distance: Optional[int] = None
    """
    The distance "as the crow flies" from the search center to this result item in
    meters.
    """

    map_view: Optional[MapView] = FieldInfo(alias="mapView", default=None)
    """
    The bounding box enclosing the geometric shape (area or line) that an individual
    result covers. `place` typed results have no `mapView`.
    """

    opening_hours: Optional[ItemOpeningHours] = FieldInfo(alias="openingHours", default=None)
    """Returns the operating hours of the place, if available."""

    position: Optional[Position] = None
    """Returns the location coordinates of the result."""

    scoring: Optional[ItemScoring] = None
    """Score of the result. A higher score indicates a closer match."""

    title: Optional[str] = None
    """The localized display name of this result item."""


class RevgeocodeRetrieveResponse(BaseModel):
    items: Optional[List[Item]] = None
    """
    The results are presented as a JSON list of candidates in ranked order
    (most-likely to least-likely) based on the matched location criteria.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .asset import (
    AssetResource,
    AsyncAssetResource,
    AssetResourceWithRawResponse,
    AsyncAssetResourceWithRawResponse,
    AssetResourceWithStreamingResponse,
    AsyncAssetResourceWithStreamingResponse,
)
from .event import (
    EventResource,
    AsyncEventResource,
    EventResourceWithRawResponse,
    AsyncEventResourceWithRawResponse,
    EventResourceWithStreamingResponse,
    AsyncEventResourceWithStreamingResponse,
)
from .location import (
    LocationResource,
    AsyncLocationResource,
    LocationResourceWithRawResponse,
    AsyncLocationResourceWithRawResponse,
    LocationResourceWithStreamingResponse,
    AsyncLocationResourceWithStreamingResponse,
)

__all__ = [
    "EventResource",
    "AsyncEventResource",
    "EventResourceWithRawResponse",
    "AsyncEventResourceWithRawResponse",
    "EventResourceWithStreamingResponse",
    "AsyncEventResourceWithStreamingResponse",
    "LocationResource",
    "AsyncLocationResource",
    "LocationResourceWithRawResponse",
    "AsyncLocationResourceWithRawResponse",
    "LocationResourceWithStreamingResponse",
    "AsyncLocationResourceWithStreamingResponse",
    "AssetResource",
    "AsyncAssetResource",
    "AssetResourceWithRawResponse",
    "AsyncAssetResourceWithRawResponse",
    "AssetResourceWithStreamingResponse",
    "AsyncAssetResourceWithStreamingResponse",
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal

import httpx

from ..types import route_report_create_params
from .._types import NOT_GIVEN, Body, Query, Headers, NotGiven
from .._utils import maybe_transform, async_maybe_transform
from .._compat import cached_property
from .._resource import SyncAPIResource, AsyncAPIResource
from .._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from .._base_client import make_request_options
from ..types.route_report_create_response import RouteReportCreateResponse

__all__ = ["RouteReportResource", "AsyncRouteReportResource"]


class RouteReportResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> RouteReportResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/nextbillion-ai/nextbillion-sdk-python#accessing-raw-response-data-eg-headers
        """
        return RouteReportResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> RouteReportResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/nextbillion-ai/nextbillion-sdk-python#with_streaming_response
        """
        return RouteReportResourceWithStreamingResponse(self)

    def create(
        self,
        *,
        key: str,
        original_shape: str,
        original_shape_type: Literal["`polyline`", "`polyline6`"],
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> RouteReportCreateResponse:
        """
        Route Report

        Args:
          key: A key is a unique identifier that is required to authenticate a request to the
              API.

          original_shape: Takes a route geometry as input and returns the route details. Accepts
              `polyline` and `polyline6` encoded geometry as input.

              **Note**: Route geometries generated from sources other than
              [NextBillion.ai](http://NextBillion.ai) services, are not supported in this
              version.

          original_shape_type: Specify the encoding type of route geometry provided in `original_shape` input.
              Please note that an error is returned when this parameter is not specified while
              an input is added to `original_shape` parameter.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._post(
            "/route_report",
            body=maybe_transform(
                {
                    "original_shape": original_shape,
                    "original_shape_type": original_shape_type,
                },
                route_report_create_params.RouteReportCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform({"key": key}, route_report_create_params.RouteReportCreateParams),
            ),
            cast_to=RouteReportCreateResponse,
        )


class AsyncRouteReportResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncRouteReportResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/nextbillion-ai/nextbillion-sdk-python#accessing-raw-response-data-eg-headers
        """
        return AsyncRouteReportResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncRouteReportResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/nextbillion-ai/nextbillion-sdk-python#with_streaming_response
        """
        return AsyncRouteReportResourceWithStreamingResponse(self)

    async def create(
        self,
        *,
        key: str,
        original_shape: str,
        original_shape_type: Literal["`polyline`", "`polyline6`"],
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> RouteReportCreateResponse:
        """
        Route Report

        Args:
          key: A key is a unique identifier that is required to authenticate a request to the
              API.

          original_shape: Takes a route geometry as input and returns the route details. Accepts
              `polyline` and `polyline6` encoded geometry as input.

              **Note**: Route geometries generated from sources other than
              [NextBillion.ai](http://NextBillion.ai) services, are not supported in this
              version.

          original_shape_type: Specify the encoding type of route geometry provided in `original_shape` input.
              Please note that an error is returned when this parameter is not specified while
              an input is added to `original_shape` parameter.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._post(
            "/route_report",
            body=await async_maybe_transform(
                {
                    "original_shape": original_shape,
                    "original_shape_type": original_shape_type,
                },
                route_report_create_params.RouteReportCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform({"key": key}, route_report_create_params.RouteReportCreateParams),
            ),
            cast_to=RouteReportCreateResponse,
        )


class RouteReportResourceWithRawResponse:
    def __init__(self, route_report: RouteReportResource) -> None:
        self._route_report = route_report

        self.create = to_raw_response_wrapper(
            route_report.create,
        )


class AsyncRouteReportResourceWithRawResponse:
    def __init__(self, route_report: AsyncRouteReportResource) -> None:
        self._route_report = route_report

        self.create = async_to_raw_response_wrapper(
            route_report.create,
        )


class RouteReportResourceWithStreamingResponse:
    def __init__(self, route_report: RouteReportResource) -> None:
        self._route_report = route_report

        self.create = to_streamed_response_wrapper(
            route_report.create,
        )


class AsyncRouteReportResourceWithStreamingResponse:
    def __init__(self, route_report: AsyncRouteReportResource) -> None:
        self._route_report = route_report

        self.create = async_to_streamed_response_wrapper(
            route_report.create,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional

from .._models import BaseModel

__all__ = [
    "RouteReportCreateResponse",
    "Mileage",
    "MileageSegment",
    "MileageSegmentCountry",
    "MileageSegmentState",
    "MileageSummary",
    "RoadSummary",
    "RoadSummarySegment",
    "RoadSummarySegmentMaxSpeed",
    "RoadSummarySegmentRoadClass",
    "RoadSummarySummary",
]


class MileageSegmentCountry(BaseModel):
    distance: Optional[float] = None
    """Represents the total distance of this segment, in meters."""

    length: Optional[int] = None
    """
    Represents a sequence of ‘n’ consecutive vertices in the route geometry starting
    from the `offset`, forming a continuous section of route with a distance
    indicated in `distance`field.
    """

    offset: Optional[int] = None
    """
    Represents the index value of the vertex of current segment's starting point in
    route geometry. First vertex in the route geometry has an offset of 0.
    """

    value: Optional[str] = None
    """Returns the name of the country in which the segment lies."""


class MileageSegmentState(BaseModel):
    distance: Optional[float] = None
    """Represents the real distance of this segment, in meters."""

    length: Optional[int] = None
    """
    Represents a sequence of ‘n’ consecutive vertices in the route geometry starting
    from the `offset`, forming a continuous section of route with a distance
    indicated in `distance`field.
    """

    offset: Optional[int] = None
    """
    Represents the index value of the vertex of current segment's starting point in
    route geometry. First vertex in the route geometry has an offset of 0.
    """

    value: Optional[str] = None
    """Returns the name of the state in which the segment lies."""


class MileageSegment(BaseModel):
    country: Optional[List[MileageSegmentCountry]] = None
    """An array of objects containing country-wise break up of the route segments.

    Each object returns the segment details of a different country.
    """

    state: Optional[List[MileageSegmentState]] = None
    """An array of objects containing state-wise break up of the route segments.

    Each object returns the segment details of a different state.
    """


class MileageSummary(BaseModel):
    country: Optional[object] = None
    """
    A break up of country-wise distances that the route covers in `key:value` pair
    format.
    """

    state: Optional[object] = None
    """
    A break up of state-wise distances that the route covers specified in
    `key:value` pair format.
    """


class Mileage(BaseModel):
    segment: Optional[MileageSegment] = None
    """
    Returns the details of road segments that the route covers in different states
    and countries.
    """

    summary: Optional[MileageSummary] = None
    """
    Returns a summary of distances that the route covers in different states and
    countries.
    """


class RoadSummarySegmentMaxSpeed(BaseModel):
    distance: Optional[int] = None
    """Returns the total distance of this segment, in meters."""

    length: Optional[int] = None
    """
    Represents a sequence of ‘n’ consecutive vertices in the route geometry starting
    from the `offset`, forming a continuous section of route where the maximum speed
    is same and is indicated in `value`.
    """

    offset: Optional[int] = None
    """
    Represents the index value of the vertex of current segment's starting point in
    route geometry. First vertex in the route geometry has an offset of 0.
    """

    value: Optional[int] = None
    """Denotes the maximum speed of this segment, in kilometers per hour.

    - A value of “-1” indicates that the speed is unlimited for this road segment. -
      A value of “0” indicates that there is no information about the maximum speed
      for this road segment.
    """


class RoadSummarySegmentRoadClass(BaseModel):
    distance: Optional[int] = None
    """Returns the total distance of this segment, in meters."""

    length: Optional[int] = None
    """
    Represents a sequence of ‘n’ consecutive vertices in the route geometry starting
    from the `offset`, forming a continuous section of route with a distance
    indicated in `distance`field.
    """

    offset: Optional[int] = None
    """
    Represents the index value of the vertex of current segment's starting point in
    route geometry. First vertex in the route geometry has an offset of 0.
    """

    value: Optional[str] = None
    """Returns the road class name to which the segment belongs."""


class RoadSummarySegment(BaseModel):
    max_speed: Optional[List[RoadSummarySegmentMaxSpeed]] = None
    """
    An array of objects returning the maximum speed of different segments that the
    route goes through.
    """

    road_class: Optional[List[RoadSummarySegmentRoadClass]] = None
    """
    An array of objects returning the details of road segments belonging to
    different road classes that the route goes through. Each object refers to a
    unique road class.
    """


class RoadSummarySummary(BaseModel):
    distance: Optional[float] = None
    """Returns the total distance of the route , in meters."""

    duration: Optional[float] = None
    """Returns the total duration of the route, in seconds."""

    has_bridge: Optional[bool] = None
    """A boolean value indicating if there are any bridges in the given route."""

    has_roundabout: Optional[bool] = None
    """A boolean value indicating if there are any roundabouts in the given route."""

    has_toll: Optional[bool] = None
    """A boolean value indicating if there are any tolls in the given route."""

    has_tunnel: Optional[bool] = None
    """A boolean value indicating if there are any tunnels in the given route."""

    road_class: Optional[object] = None
    """
    An object with details about the different types of road classes that the route
    goes through. Distance traversed on a given road class is also returned. The
    contents of this object follow the `key:value` pair format.
    """

    toll_distance: Optional[float] = None
    """Returns the total distance travelled on toll roads.

    This field is present in the response only when the `has_toll` property is true.
    """


class RoadSummary(BaseModel):
    segment: Optional[RoadSummarySegment] = None
    """Returns the segment-wise road class and max speed information of the route."""

    summary: Optional[RoadSummarySummary] = None
    """
    Returns an overview of the route with information about trip distance, duration
    and road class details among others.
    """


class RouteReportCreateResponse(BaseModel):
    geometry: Optional[List[str]] = None
    """An array of objects returning encoded geometry of the routes.

    Each object represents an individual route in the input.
    """

    mileage: Optional[List[Mileage]] = None
    """
    Returns the details of route segments in each state or country that the route
    passes through. Each object represents an individual route in the input request.
    """

    msg: Optional[str] = None
    """Displays the error message in case of a failed request or operation.

    Please note that this parameter is not returned in the response in case of a
    successful request.
    """

    road_summary: Optional[List[RoadSummary]] = None
    """
    An array of objects returning a summary of the route with information about
    tolls, bridges, tunnels, segments, maximum speeds and more. Each array
    represents an individual route in the input request.
    """

    status: Optional[str] = None
    """A string indicating the state of the response.

    On normal responses, the value will be `Ok`. Indicative HTTP error codes are
    returned for different errors. See the
    [**API Errors Codes**](https://app.reapi.com/ws/hmx8aL45B5jjrJa8/p/vNNilNksLVz675pI/s/ealJmVGjTQv4x5Wi/edit/path/VYzo7gOlRsQQZo0U#api-error-codes)
    section below for more information.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Iterable
from typing_extensions import Literal, Required, TypedDict

__all__ = ["DocumentTemplateContentRequestParam", "Meta", "MetaOption", "Validation"]


class MetaOption(TypedDict, total=False):
    label: Required[str]
    """Specify the label or name for the option."""

    value: Required[str]
    """Specify the value associated with the option.

    This value will be submitted when the option is checked in the Driver app.
    """


class Meta(TypedDict, total=False):
    options: Required[Iterable[MetaOption]]
    """
    An array of objects to define options for a `multi_choices` or `single_choice`
    type document field. Each object represents one option.
    """


class Validation(TypedDict, total=False):
    max: int
    """Specifies the maximum allowed value for `number` type document field.

    Input values must be less than or equal to this threshold.
    """

    max_items: int
    """
    Specifies the maximum number of items for `multi_choices`, `photos` type
    document fields. The number of provided input items must be less than or equal
    to this threshold.
    """

    min: int
    """Specifies the minimum allowed value for `number` type document field.

    Input values must be greater than or equal to this threshold.
    """

    min_items: int
    """
    Specifies the minimum number of items for `multi_choices`, `photos` type
    document fields. The number of provided input items must be greater than or
    equal to this threshold.
    """


class DocumentTemplateContentRequestParam(TypedDict, total=False):
    label: Required[str]
    """Specify the label or the name of the field.

    The `label` specified here can be used as field name when rendering the document
    in the Driver app.
    """

    type: Required[
        Literal[
            "`string`",
            "`number`",
            "`date_time`",
            "`photos`",
            "`multi_choices`",
            "`signature`",
            "`barcode`",
            "`single_choice`",
        ]
    ]
    """Specify the data type of the field.

    It corresponds to the type of information that the driver needs to collect.
    """

    meta: Meta
    """
    An object to define additional information required for `single_choice` or
    `multi_choices` type document items.
    """

    name: str
    """Specify the name of the document field.

    A field's`name` can be used for internal references to the document field.
    """

    required: bool
    """Specify if it is mandatory to fill the field. Default value is false."""

    validation: Validation
    """Specify the validation rules for the field.

    This can be used to enforce data quality and integrity checks. For example, if
    the field is a number type, `validation` can define constraints like minimum /
    maximum number values.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal, Required, TypedDict

__all__ = ["NavigationRetrieveRouteParams"]


class NavigationRetrieveRouteParams(TypedDict, total=False):
    key: Required[str]
    """
    A key is a unique identifier that is required to authenticate a request to the
    API.
    """

    altcount: int
    """Sets the number of alternative routes to return.

    It is effective only when "alternatives" is "true". Please note that adding
    alternative route count does not guarantee matching number of routes to be
    returned if potential alternative routes do not exist.
    """

    alternatives: bool
    """When "true" the API will return alternate routes.

    The "alternatives" is effective only when there are no "waypoints" included in
    the request. You can set the number of alternate routes to be returned in the
    "altcount" property.
    """

    approaches: Literal["`unrestricted`", "`curb`"]
    """
    A semicolon-separated list indicating the side of the road from which to
    approach "waypoints" in a requested route. When set to "unrestricted" a route
    can arrive at the waypoint from either side of the road and when set to "curb"
    the route will arrive at the waypoint on the driving side of the region. Please
    note the number of values provided must be one more than the number of
    "waypoints". The last value of "approaches" will determine the approach for the
    "destination". However, you can skip a coordinate and show its position in the
    list with the ";" separator.
    """

    avoid: Literal["toll", "ferry", "highway", "none"]
    """Setting this will ensure the route avoids ferries, tolls, highways or nothing.

    Multiple values should be separated by a pipe (|). If "none" is provided along
    with other values, an error is returned as a valid route is not feasible. Please
    note that when this parameter is not provided in the input, ferries are set to
    be avoided by default. When this parameter is provided, only the mentioned
    objects are avoided.
    """

    bearings: str
    """
    Limits the search to road segments with given bearing, in degrees, towards true
    north in clockwise direction. Each "bearings" should be in the format of
    "degree,range", where the "degree" should be a value between \\[[0, 360\\]] and
    "range" should be a value between \\[[0, 180\\]]. Please note that the number of
    "bearings" should be two more than the number of "waypoints". This is to account
    for the bearing of "origin" and "destination". If a route can approach a
    "waypoint" or the "destination" from any direction, the bearing for that point
    can be specified as "0,180".
    """

    destination: str
    """\"destination" is the ending point of your route.

    Ensure that the "destination" is a routable land location. Please note that this
    parameter is mandatory if the "original_shape" parameter is not given.
    """

    geometry: Literal["`polyline`", "`polyline6`", "`geojson`"]
    """Sets the output format of the route geometry in the response.

    On providing “polyline“ and “polyline6“ as input, respective encoded geometry is
    returned. However, when “geojson“ is provided as the input value, “polyline“
    encoded geometry is returned in the response along with the geojson details of
    the route.
    """

    lang: str
    """
    Select the language to be used for result rendering from a list of
    [BCP 47](https://en.wikipedia.org/wiki/IETF_language_tag) compliant language
    codes.
    """

    mode: Literal["`car`", "`truck`"]
    """Set which driving mode the service should use to determine a route.

    For example, if you use "car", the API will return a route that a car can take.
    Using "truck" will return a route a truck can use, taking into account
    appropriate truck routing restrictions.

    When "mode=truck", following are the default dimensions that are used:

    \\-- truck_height = 214 centimeters

    \\-- truck_width = 183 centimeters

    \\-- truck_length = 519 centimeters

    \\-- truck_weight = 5000 kg

    Please use the Navigation Flexible version if you want to use custom truck
    dimensions.

    Note: Only the "car" profile is enabled by default. Please note that customized
    profiles (including "truck") might not be available for all regions. Please
    contact your [NextBillion.ai](http://NextBillion.ai) account manager, sales
    representative or reach out at
    [support@nextbillion.ai](mailto:support@nextbillion.ai) in case you need
    additional profiles.
    """

    origin: str
    """\"origin" is the starting point of your route.

    Ensure that "origin" is a routable land location. Please note that this
    parameter is mandatory if the geometry parameter is not given.
    """

    original_shape: str
    """
    Takes a route geometry as input and returns the navigation information for that
    route. Accepts "polyline" and "polyline6" encoded geometry as input.
    "original_shape_type" becomes mandatory when "original_shape" is used. If this
    parameter is provided, the only other parameters which will be considered are
    "original_shape_type", "lang", "geometry". The rest of the parameters in the
    input request will be ignored. Please note overview verbosity will always be
    "full" when using this parameter.
    """

    original_shape_type: Literal["polyline", "polyline6"]
    """
    Specify the encoding format of route geometry provided in the request using
    "original_shape" parameter. Please note that an error is returned when this
    parameter is not specified while an input is added to "original_shape"
    parameter.
    """

    overview: Literal["`full`", "`simplified`", "`false`"]
    """Specify the verbosity of route geometry.

    When set to "full", the most detailed geometry available is returned. When set
    to "simplified", a simplified version of the full geometry is returned. No
    overview geometry is returned when set to "false".
    """

    waypoints: str
    """
    "waypoints" are coordinates along the route between the "origin" and
    "destination". It is a pipe-separated list of coordinate pairs. Please note that
    the route returned will arrive at the "waypoints" in the sequence they are
    provided in the input request. Please note that the maximum number of waypoints
    that can be provided in a single request is 50 when using GET method and 200
    with POST method.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import List
from typing_extensions import Literal

import httpx

from .v2 import (
    V2Resource,
    AsyncV2Resource,
    V2ResourceWithRawResponse,
    AsyncV2ResourceWithRawResponse,
    V2ResourceWithStreamingResponse,
    AsyncV2ResourceWithStreamingResponse,
)
from ...types import optimization_compute_params, optimization_re_optimize_params
from ..._types import NOT_GIVEN, Body, Query, Headers, NotGiven
from ..._utils import maybe_transform, async_maybe_transform
from ..._compat import cached_property
from ..._resource import SyncAPIResource, AsyncAPIResource
from ..._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ..._base_client import make_request_options
from .driver_assignment import (
    DriverAssignmentResource,
    AsyncDriverAssignmentResource,
    DriverAssignmentResourceWithRawResponse,
    AsyncDriverAssignmentResourceWithRawResponse,
    DriverAssignmentResourceWithStreamingResponse,
    AsyncDriverAssignmentResourceWithStreamingResponse,
)
from ...types.post_response import PostResponse
from ...types.optimization_compute_response import OptimizationComputeResponse

__all__ = ["OptimizationResource", "AsyncOptimizationResource"]


class OptimizationResource(SyncAPIResource):
    @cached_property
    def driver_assignment(self) -> DriverAssignmentResource:
        return DriverAssignmentResource(self._client)

    @cached_property
    def v2(self) -> V2Resource:
        return V2Resource(self._client)

    @cached_property
    def with_raw_response(self) -> OptimizationResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/nextbillion-ai/nextbillion-sdk-python#accessing-raw-response-data-eg-headers
        """
        return OptimizationResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> OptimizationResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/nextbillion-ai/nextbillion-sdk-python#with_streaming_response
        """
        return OptimizationResourceWithStreamingResponse(self)

    def compute(
        self,
        *,
        coordinates: str,
        key: str,
        approaches: Literal["`unrestricted`", "`curb`"] | NotGiven = NOT_GIVEN,
        destination: Literal["`any`", "`last`"] | NotGiven = NOT_GIVEN,
        geometries: Literal["`polyline`", "`polyline6`", "`geojson`"] | NotGiven = NOT_GIVEN,
        mode: Literal["`car`", "`truck`"] | NotGiven = NOT_GIVEN,
        roundtrip: bool | NotGiven = NOT_GIVEN,
        source: Literal["`any`", "`first`"] | NotGiven = NOT_GIVEN,
        with_geometry: bool | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> OptimizationComputeResponse:
        """
        Nextbillion.ai Optimization API computes and returns an optimized route between
        an origin and destination which have multiple stop points in between. With
        NextBillion.ai's Route Optimization API you get.

        Optimized routing between way points

        Highly accurate ETAs with customized routes

        Roundtrip optimization with customized destinations

        A list of all parameters is specified in the next section.

        Args:
          coordinates: This is a pipe-separated list of coordinates.

              Minimum 3 pairs of coordinates and Maximum 12 pairs of coordinates are allowed.

          key: A key is a unique identifier that is required to authenticate a request to the
              API.

          approaches: A semicolon-separated list indicating the side of the road from which to
              approach waypoints in a requested route. If provided, the number of `approaches`
              must be the same as the number of `coordinates`. However, you can skip a
              coordinate and show its position in the list with the `;` separator.

          destination: Specify the destination coordinate of the returned route. If the input is
              `last`, the last coordinate will be the destination.

          geometries: Sets the output format of the route geometry in the response.

              On providing `polyline` and `polyline6` as input, respective encoded geometry is
              returned. However, when `geojson` is provided as the input value, `polyline`
              encoded geometry is returned in the response along with a geojson details of the
              route.

          mode: Set which driving mode the service should use to determine a route. For example,
              if you use "car", the API will return a route that a car can take. Using "truck"
              will return a route a truck can use, taking into account appropriate truck
              routing restrictions.

              When "mode=truck", following are the default dimensions that are used:

              \\-- truck_height = 214 centimeters

              \\-- truck_width = 183 centimeters

              \\-- truck_length = 519 centimeters

              \\-- truck_weight = 5000 kg

              Please use the Directions Flexible version if you want to use custom truck
              dimensions.

              Note: Only the "car" profile is enabled by default. Please note that customized
              profiles (including "truck") might not be available for all regions. Please
              contact your [NextBillion.ai](http://NextBillion.ai) account manager, sales
              representative or reach out at
              [support@nextbillion.ai](mailto:support@nextbillion.ai) in case you need
              additional profiles.

          roundtrip: Indicates whether the returned route is a roundtrip.

          source: The coordinate at which to start the returned route. If this is not configured,
              the return route’s destination will be the first coordinate.

          with_geometry: Indicates whether the return geometry should be computed or not.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get(
            "/optimization/json",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "coordinates": coordinates,
                        "key": key,
                        "approaches": approaches,
                        "destination": destination,
                        "geometries": geometries,
                        "mode": mode,
                        "roundtrip": roundtrip,
                        "source": source,
                        "with_geometry": with_geometry,
                    },
                    optimization_compute_params.OptimizationComputeParams,
                ),
            ),
            cast_to=OptimizationComputeResponse,
        )

    def re_optimize(
        self,
        *,
        key: str,
        existing_request_id: str,
        job_changes: optimization_re_optimize_params.JobChanges | NotGiven = NOT_GIVEN,
        locations: List[str] | NotGiven = NOT_GIVEN,
        shipment_changes: optimization_re_optimize_params.ShipmentChanges | NotGiven = NOT_GIVEN,
        vehicle_changes: optimization_re_optimize_params.VehicleChanges | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> PostResponse:
        """
        Re-optimization

        Args:
          key: A key is a unique identifier that is required to authenticate a request to the
              API.

          existing_request_id: Specify the unique request ID that needs to be re-optimized.

          job_changes: This section gathers information on modifications to the number of jobs or their
              individual requirements for re-optimization. Any job from the original solution
              not specified here will be re-planned without alteration during the
              re-optimization process.

          locations: Provide the list of locations to be used during re-optimization process. Please
              note that

              - Providing the location input overwrites the list of locations used in the
                original request.
              - The location_indexes associated with all tasks and vehicles (both from the
                original and new re-optimization input requests) will follow the updated list
                of locations.

              As a best practice:

              1.  Don't provide the `locations` input when re-optimizing, if the original set
                  contains all the required location coordinates.
              2.  If any new location coordinates are required for re-optimization, copy the
                  full, original location list and update it in the following manner before
                  adding it to the re-optimization input:

                  1.  Ensure to not update the indexes of locations which just need to be
                      "modified".
                  2.  Add new location coordinates towards the end of the list.

          shipment_changes: This section gathers information on modifications to the number of shipments or
              their individual requirements for re-optimization. Any shipment from the
              original solution not specified here will be re-planned without alteration
              during the re-optimization process.

          vehicle_changes: This section gathers information on modifications to the number of vehicles or
              individual vehicle configurations for re-optimizing an existing solution. Any
              vehicle from the original solution not specified here will be reused without
              alteration during the re-optimization process.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._post(
            "/optimization/re_optimization",
            body=maybe_transform(
                {
                    "existing_request_id": existing_request_id,
                    "job_changes": job_changes,
                    "locations": locations,
                    "shipment_changes": shipment_changes,
                    "vehicle_changes": vehicle_changes,
                },
                optimization_re_optimize_params.OptimizationReOptimizeParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform({"key": key}, optimization_re_optimize_params.OptimizationReOptimizeParams),
            ),
            cast_to=PostResponse,
        )


class AsyncOptimizationResource(AsyncAPIResource):
    @cached_property
    def driver_assignment(self) -> AsyncDriverAssignmentResource:
        return AsyncDriverAssignmentResource(self._client)

    @cached_property
    def v2(self) -> AsyncV2Resource:
        return AsyncV2Resource(self._client)

    @cached_property
    def with_raw_response(self) -> AsyncOptimizationResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/nextbillion-ai/nextbillion-sdk-python#accessing-raw-response-data-eg-headers
        """
        return AsyncOptimizationResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncOptimizationResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/nextbillion-ai/nextbillion-sdk-python#with_streaming_response
        """
        return AsyncOptimizationResourceWithStreamingResponse(self)

    async def compute(
        self,
        *,
        coordinates: str,
        key: str,
        approaches: Literal["`unrestricted`", "`curb`"] | NotGiven = NOT_GIVEN,
        destination: Literal["`any`", "`last`"] | NotGiven = NOT_GIVEN,
        geometries: Literal["`polyline`", "`polyline6`", "`geojson`"] | NotGiven = NOT_GIVEN,
        mode: Literal["`car`", "`truck`"] | NotGiven = NOT_GIVEN,
        roundtrip: bool | NotGiven = NOT_GIVEN,
        source: Literal["`any`", "`first`"] | NotGiven = NOT_GIVEN,
        with_geometry: bool | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> OptimizationComputeResponse:
        """
        Nextbillion.ai Optimization API computes and returns an optimized route between
        an origin and destination which have multiple stop points in between. With
        NextBillion.ai's Route Optimization API you get.

        Optimized routing between way points

        Highly accurate ETAs with customized routes

        Roundtrip optimization with customized destinations

        A list of all parameters is specified in the next section.

        Args:
          coordinates: This is a pipe-separated list of coordinates.

              Minimum 3 pairs of coordinates and Maximum 12 pairs of coordinates are allowed.

          key: A key is a unique identifier that is required to authenticate a request to the
              API.

          approaches: A semicolon-separated list indicating the side of the road from which to
              approach waypoints in a requested route. If provided, the number of `approaches`
              must be the same as the number of `coordinates`. However, you can skip a
              coordinate and show its position in the list with the `;` separator.

          destination: Specify the destination coordinate of the returned route. If the input is
              `last`, the last coordinate will be the destination.

          geometries: Sets the output format of the route geometry in the response.

              On providing `polyline` and `polyline6` as input, respective encoded geometry is
              returned. However, when `geojson` is provided as the input value, `polyline`
              encoded geometry is returned in the response along with a geojson details of the
              route.

          mode: Set which driving mode the service should use to determine a route. For example,
              if you use "car", the API will return a route that a car can take. Using "truck"
              will return a route a truck can use, taking into account appropriate truck
              routing restrictions.

              When "mode=truck", following are the default dimensions that are used:

              \\-- truck_height = 214 centimeters

              \\-- truck_width = 183 centimeters

              \\-- truck_length = 519 centimeters

              \\-- truck_weight = 5000 kg

              Please use the Directions Flexible version if you want to use custom truck
              dimensions.

              Note: Only the "car" profile is enabled by default. Please note that customized
              profiles (including "truck") might not be available for all regions. Please
              contact your [NextBillion.ai](http://NextBillion.ai) account manager, sales
              representative or reach out at
              [support@nextbillion.ai](mailto:support@nextbillion.ai) in case you need
              additional profiles.

          roundtrip: Indicates whether the returned route is a roundtrip.

          source: The coordinate at which to start the returned route. If this is not configured,
              the return route’s destination will be the first coordinate.

          with_geometry: Indicates whether the return geometry should be computed or not.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._get(
            "/optimization/json",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform(
                    {
                        "coordinates": coordinates,
                        "key": key,
                        "approaches": approaches,
                        "destination": destination,
                        "geometries": geometries,
                        "mode": mode,
                        "roundtrip": roundtrip,
                        "source": source,
                        "with_geometry": with_geometry,
                    },
                    optimization_compute_params.OptimizationComputeParams,
                ),
            ),
            cast_to=OptimizationComputeResponse,
        )

    async def re_optimize(
        self,
        *,
        key: str,
        existing_request_id: str,
        job_changes: optimization_re_optimize_params.JobChanges | NotGiven = NOT_GIVEN,
        locations: List[str] | NotGiven = NOT_GIVEN,
        shipment_changes: optimization_re_optimize_params.ShipmentChanges | NotGiven = NOT_GIVEN,
        vehicle_changes: optimization_re_optimize_params.VehicleChanges | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> PostResponse:
        """
        Re-optimization

        Args:
          key: A key is a unique identifier that is required to authenticate a request to the
              API.

          existing_request_id: Specify the unique request ID that needs to be re-optimized.

          job_changes: This section gathers information on modifications to the number of jobs or their
              individual requirements for re-optimization. Any job from the original solution
              not specified here will be re-planned without alteration during the
              re-optimization process.

          locations: Provide the list of locations to be used during re-optimization process. Please
              note that

              - Providing the location input overwrites the list of locations used in the
                original request.
              - The location_indexes associated with all tasks and vehicles (both from the
                original and new re-optimization input requests) will follow the updated list
                of locations.

              As a best practice:

              1.  Don't provide the `locations` input when re-optimizing, if the original set
                  contains all the required location coordinates.
              2.  If any new location coordinates are required for re-optimization, copy the
                  full, original location list and update it in the following manner before
                  adding it to the re-optimization input:

                  1.  Ensure to not update the indexes of locations which just need to be
                      "modified".
                  2.  Add new location coordinates towards the end of the list.

          shipment_changes: This section gathers information on modifications to the number of shipments or
              their individual requirements for re-optimization. Any shipment from the
              original solution not specified here will be re-planned without alteration
              during the re-optimization process.

          vehicle_changes: This section gathers information on modifications to the number of vehicles or
              individual vehicle configurations for re-optimizing an existing solution. Any
              vehicle from the original solution not specified here will be reused without
              alteration during the re-optimization process.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._post(
            "/optimization/re_optimization",
            body=await async_maybe_transform(
                {
                    "existing_request_id": existing_request_id,
                    "job_changes": job_changes,
                    "locations": locations,
                    "shipment_changes": shipment_changes,
                    "vehicle_changes": vehicle_changes,
                },
                optimization_re_optimize_params.OptimizationReOptimizeParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform(
                    {"key": key}, optimization_re_optimize_params.OptimizationReOptimizeParams
                ),
            ),
            cast_to=PostResponse,
        )


class OptimizationResourceWithRawResponse:
    def __init__(self, optimization: OptimizationResource) -> None:
        self._optimization = optimization

        self.compute = to_raw_response_wrapper(
            optimization.compute,
        )
        self.re_optimize = to_raw_response_wrapper(
            optimization.re_optimize,
        )

    @cached_property
    def driver_assignment(self) -> DriverAssignmentResourceWithRawResponse:
        return DriverAssignmentResourceWithRawResponse(self._optimization.driver_assignment)

    @cached_property
    def v2(self) -> V2ResourceWithRawResponse:
        return V2ResourceWithRawResponse(self._optimization.v2)


class AsyncOptimizationResourceWithRawResponse:
    def __init__(self, optimization: AsyncOptimizationResource) -> None:
        self._optimization = optimization

        self.compute = async_to_raw_response_wrapper(
            optimization.compute,
        )
        self.re_optimize = async_to_raw_response_wrapper(
            optimization.re_optimize,
        )

    @cached_property
    def driver_assignment(self) -> AsyncDriverAssignmentResourceWithRawResponse:
        return AsyncDriverAssignmentResourceWithRawResponse(self._optimization.driver_assignment)

    @cached_property
    def v2(self) -> AsyncV2ResourceWithRawResponse:
        return AsyncV2ResourceWithRawResponse(self._optimization.v2)


class OptimizationResourceWithStreamingResponse:
    def __init__(self, optimization: OptimizationResource) -> None:
        self._optimization = optimization

        self.compute = to_streamed_response_wrapper(
            optimization.compute,
        )
        self.re_optimize = to_streamed_response_wrapper(
            optimization.re_optimize,
        )

    @cached_property
    def driver_assignment(self) -> DriverAssignmentResourceWithStreamingResponse:
        return DriverAssignmentResourceWithStreamingResponse(self._optimization.driver_assignment)

    @cached_property
    def v2(self) -> V2ResourceWithStreamingResponse:
        return V2ResourceWithStreamingResponse(self._optimization.v2)


class AsyncOptimizationResourceWithStreamingResponse:
    def __init__(self, optimization: AsyncOptimizationResource) -> None:
        self._optimization = optimization

        self.compute = async_to_streamed_response_wrapper(
            optimization.compute,
        )
        self.re_optimize = async_to_streamed_response_wrapper(
            optimization.re_optimize,
        )

    @cached_property
    def driver_assignment(self) -> AsyncDriverAssignmentResourceWithStreamingResponse:
        return AsyncDriverAssignmentResourceWithStreamingResponse(self._optimization.driver_assignment)

    @cached_property
    def v2(self) -> AsyncV2ResourceWithStreamingResponse:
        return AsyncV2ResourceWithStreamingResponse(self._optimization.v2)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional

from pydantic import Field as FieldInfo

from .._models import BaseModel

__all__ = ["IsochroneComputeResponse", "Feature", "FeatureGeometry", "FeatureProperties"]


class FeatureGeometry(BaseModel):
    coordinates: Optional[List[float]] = None
    """
    An array of coordinate points, in [longitude,latitude] format representing the
    isochrone contour line.
    """

    type: Optional[str] = None
    """Type of the geoJSON geometry."""


class FeatureProperties(BaseModel):
    color: Optional[str] = None
    """The hex code of the color of the isochrone contour line"""

    contour: Optional[float] = None
    """The value of the metric used in this contour.

    See the `metric` property to determine whether this is a `time` or `distance`
    contour. When the `metric` is `time` this value denotes the travel time in
    minutes and when the `metric` is `distance` this value denotes the travel
    distance in kilometers.
    """

    fill: Optional[str] = None
    """The hex code for the fill color of the isochrone contour line."""

    fill_color: Optional[str] = FieldInfo(alias="fillColor", default=None)
    """The hex code for the fill color of the isochrone contour line"""

    fill_opacity: Optional[float] = FieldInfo(alias="fillOpacity", default=None)
    """The fill opacity for the isochrone contour line.

    It is a float value starting from 0.0 with a max value of 1.0. Higher number
    indicates a higher fill opacity.
    """

    metric: Optional[str] = None
    """The metric that the contour represents - either `distance` or `time`"""

    opacity: Optional[float] = None
    """The opacity of the isochrone contour line.

    It is a float value starting from 0.0 with a max value of 1.0. Higher number
    indicates a higher line opacity
    """


class Feature(BaseModel):
    geometry: Optional[FeatureGeometry] = None
    """
    A [GeoJSON geometry](https://datatracker.ietf.org/doc/html/rfc7946#page-7)
    object with details of the contour line.
    """

    properties: Optional[FeatureProperties] = None
    """An object with details of how the isochrone contour can be drawn on a map."""

    type: Optional[str] = None
    """Type of the GeoJSON object.

    Its value is `Feature` as per the
    [GeoJSON standard](https://datatracker.ietf.org/doc/html/rfc7946#section-1.4)
    object.
    """


class IsochroneComputeResponse(BaseModel):
    features: Optional[List[Feature]] = None
    """
    A
    [GeoJSON FeatureCollection](https://datatracker.ietf.org/doc/html/rfc7946#section-3.3)
    object with details of the isochrone contours. Each `feature` object in this
    collection represents an isochrone.
    """

    msg: Optional[str] = None
    """Displays the error message in case of a failed request or operation.

    Please note that this parameter is not returned in the response in case of a
    successful request.
    """

    status: Optional[str] = None
    """A string indicating the state of the response.

    On normal responses, the value will be `Ok`. Indicative HTTP error codes are
    returned for different errors. See the [API Errors Codes](#api-error-codes)
    section below for more information.
    """

    type: Optional[str] = None
    """Type of the GeoJSON object.

    As prescribed in
    [GeoJSON standard](https://datatracker.ietf.org/doc/html/rfc7946#section-1.4),
    its value is `FeatureCollection` as the `feature` property contains a list of
    geoJSON feature objects.
    """
